
# Substream-Centric Maximum Matchings on FPGA 

This repository contains the code to the paper "Substream-Centric Maximum Matchings on FPGA" by Maciej Besta et al. (maciej.besta@inf.ethz.ch )

## Getting Started

These following section will provide an overview of the code and get you started to run the code on your own machine.

### Project Structure

*  The C++ code is available at /streaming. The main parts are the following:
    * /streaming/io
Code to manage input and output; especially reading different input file formats, which describe graphs.
    * /streaming/runconfig
Reader for run-configurations, which describe parameters for testing and performance measurements.
    * /streaming/mwmsolver
Different solvers (algorithms) for maximum weighted matching.
    * /streaming/util
    Util functions used in various classes.
    * /streaming/output
Output directory to which all output data (measurements, configurations etc) is written to.
- The RapidJSON C++ library to parse .json files (https://github.com/Tencent/rapidjson/) is cloned in /rapidjson.
- A copy of the Lemon Graph Library (https://lemon.cs.elte.hu/trac/lemon) is cloned in /lemon.
- The verilog code of the MWM implementation is stored under /fpga.

### Prerequisites

The following software is required to compile the C++ code

- CMake: CMake version >= 2.8.
- C++ Compiler: A C++ compiler of your choice.
- Cenatur: Clone and install Centaur at ~/centaur (see https://github.com/fpgasystems/Centaur). Please contact Muhsen Owaida (muhsen.owaida@inf.ethz.ch) for a recent copy.
- Lemon Graph Library: Install the Lemon Graph Library. See the installation notes in lemon/lemon-1.3.1/INSTALL.
If you want to install it locally (for example to /local/lemon), replace step 3 of the INSTALL notes by:
cmake .. -DCMAKE_INSTALL_PREFIX=/local/lemon
Do not forget to change the path in the Makefile in the variable "COMPILE_FLAGS_USER_LOCAL".
- Intel Accelerator Abstraction Layer. We assume it is installed at /opt/aalsdk for Harp2 or /opt/aal for the simulator. If not installed, use omit the "-DFPGA_FLAG=True" and "-DASEAFU=True" flag shown in the Installing section. Note however that without the AAL, you are not able to call the FPGA MWM algorithm of Crouch and Stubbs.

The following software is required to compile the Verilog code
- Centaur: See above.
- DoppioDB: See https://github.com/fpgasystems/doppiodb. Please contact Muhsen Owaida (muhsen.owaida@inf.ethz.ch) for a recent copy.

### Installing

In the following, we assume the required software is installed as described.

To compile the C++ code for the AAL simulator, run the following steps in the folder /streaming

```
mkdir build
cd build
cmake .. -DFPGA_FLAG=True -DASEAFU=True -DLEMON_INSTALLED=True
make -j4
```

To compile the Verilog code, we refer to the Jenkins example build plan (Jenkinsfile) which outlines how the code can be compiled to the bitstream using Centaur and DoppioDB.

