	`include "../afu_defines.vh"

module AFU #(parameter AFU_OPERATOR         = `UNDEF_AFU,          // AFU Common parameters
             parameter MAX_AFU_CONFIG_WIDTH = 1536,
             parameter USER_RD_TAG          = `AFU_TAG,
             parameter USER_WR_TAG          = `AFU_TAG,
             parameter AFU_CLK              = `BASE_CLK,
             //------ AFU Specific Paramters ----//
             parameter DATA_WIDTH_IN    = 4,
             parameter DATA_WIDTH_OUT   = 4,
             parameter USER_AFU_PARAMETER1 = 1,
             parameter USER_AFU_PARAMETER2 = 1
) (
    input   wire                                   clk400,
    input   wire                                   clk200,
    input   wire                                   rst_n,
    //-------------------------------------------------//
    input   wire                                   start_um,
    input   wire [MAX_AFU_CONFIG_WIDTH-1:0]        um_params,
    output  wire                                   um_done,
    output  wire [`NUM_USER_STATE_COUNTERS*32-1:0] um_state_counters,
    output  wire                                   um_state_counters_valid,
    // TX RD
    output  wire [`CL_USR_VIRT_ADDRESS_WIDTH-1:0]  um_tx_rd_addr,
    output  wire [USER_RD_TAG-1:0]                 um_tx_rd_tag,
    output  wire                                   um_tx_rd_valid,
    input   wire                                   um_tx_rd_ready,
    input   wire                                   um_tx_rd_ready4,
    // TX WR
    output  wire [`CL_USR_VIRT_ADDRESS_WIDTH-1:0]  um_tx_wr_addr,
    output  wire [USER_WR_TAG-1:0]                 um_tx_wr_tag,
    output  wire                                   um_tx_wr_valid,
    output  wire [511:0]                           um_tx_data,
    input   wire                                   um_tx_wr_ready,
    input   wire                                   um_tx_wr_ready4,
    // RX RD
    input   wire [USER_RD_TAG-1:0]                 um_rx_rd_tag,
    input   wire [511:0]                           um_rx_data,
    input   wire                                   um_rx_rd_valid,
    output  wire                                   um_rx_rd_ready,
    // RX WR 
    input   wire                                   um_rx_wr_valid,
    input   wire [USER_WR_TAG-1:0]                 um_rx_wr_tag
);

wire   clk;
generate 
    if(AFU_CLK == 200) begin
        assign clk = clk200;
    end
    else begin 
        assign clk = clk400;
    end
endgenerate


generate
    if( AFU_OPERATOR == `MWM_CROUCH_FIFO_BRAM_AFU) begin

			mwm_crouch_bram_fifo  mwm_crouch_bram_fifo (
		.clk                                (clk),
		.rst_n                              (rst_n),
		//-------------------------------------------------//
		.start_um                           (start_um),
		.um_params                          (um_params[511:0]),
		.um_done                            (um_done),
		// User Module TX RD
		.um_tx_rd_addr                      (um_tx_rd_addr),
		.um_tx_rd_tag                       (um_tx_rd_tag),
		.um_tx_rd_valid                     (um_tx_rd_valid),
		.um_tx_rd_ready                     (um_tx_rd_ready),
		// User Module TX WR
		.um_tx_wr_addr                      (um_tx_wr_addr),
		.um_tx_wr_tag                       (um_tx_wr_tag),
		.um_tx_wr_valid                     (um_tx_wr_valid),
		.um_tx_data                         (um_tx_data),
		.um_tx_wr_ready                     (um_tx_wr_ready),
		// User Module RX RD
		.um_rx_rd_tag                       (um_rx_rd_tag),
		.um_rx_data                         (um_rx_data),
		.um_rx_rd_valid                     (um_rx_rd_valid),
		.um_rx_rd_ready                     (um_rx_rd_ready),
		// User Module RX WR 
		.um_rx_wr_valid                     (um_rx_wr_valid),
		.um_rx_wr_tag                       (um_rx_wr_tag)
		);

		assign um_state_counters_valid = 1'b0;
	end

    else begin
        assign um_tx_rd_valid          = 1'b0;
        assign um_tx_wr_valid          = 1'b0;
        assign um_rd_done              = 1'b0;
        assign um_wr_done              = 1'b0;
        assign um_state_counters_valid = 1'b0;
    end
endgenerate





endmodule // AFU
