
module length_sorter_interm8 #(parameter K_BITS = 4, parameter SORTER_QUEUE_LENGTH_BITS = 5, parameter PTR_QUEUE_LENGTH_BITS = 5  ) (
	//---------------------- GENERAL VARIABLES---------------------------//
	input   wire                                   	clk,							// clock
    input   wire                                   	rst_n,							// Stays low as long as AFU is idle
	input wire 										um_done,						// true, if we are done processing (global)
	input wire 										um_activated,					// true, if the afu is activated
	input wire 										edge_requester_done,

	input wire[K_BITS-1:0]							queue_1_k,
	input wire[SORTER_QUEUE_LENGTH_BITS:0]									queue_1_edge_length,
	input wire										queue_1_ptr_length,

	input wire[K_BITS-1:0]							queue_2_k,
	input wire[SORTER_QUEUE_LENGTH_BITS:0]									queue_2_edge_length,
	input wire										queue_2_ptr_length,

	input wire[K_BITS-1:0]							queue_3_k,
	input wire[SORTER_QUEUE_LENGTH_BITS:0]									queue_3_edge_length,
	input wire										queue_3_ptr_length,

	input wire[K_BITS-1:0]							queue_4_k,
	input wire[SORTER_QUEUE_LENGTH_BITS:0]									queue_4_edge_length,
	input wire										queue_4_ptr_length,

	input wire[K_BITS-1:0]							queue_5_k,
	input wire[SORTER_QUEUE_LENGTH_BITS:0]									queue_5_edge_length,
	input wire										queue_5_ptr_length,

	input wire[K_BITS-1:0]							queue_6_k,
	input wire[SORTER_QUEUE_LENGTH_BITS:0]									queue_6_edge_length,
	input wire										queue_6_ptr_length,

	input wire[K_BITS-1:0]							queue_7_k,
	input wire[SORTER_QUEUE_LENGTH_BITS:0]									queue_7_edge_length,
	input wire										queue_7_ptr_length,

	input wire[K_BITS-1:0]							queue_8_k,
	input wire[SORTER_QUEUE_LENGTH_BITS:0]									queue_8_edge_length,
	input wire										queue_8_ptr_length,

	output reg[K_BITS-1:0]							out_k,
	output reg[SORTER_QUEUE_LENGTH_BITS:0]									out_edge_length,
	output reg										out_ptr_length
);

always @(posedge clk) begin
    if (~rst_n | um_done | ~um_activated) begin
			out_k <= 0;
			out_edge_length <= 0;
			out_ptr_length <= 0;
			// out_k_valid <= 0;
	end
	else begin
			if( ( queue_1_edge_length <= queue_2_edge_length || queue_2_ptr_length == 0 ) &&
				( queue_1_edge_length <= queue_3_edge_length || queue_3_ptr_length == 0 ) &&
				( queue_1_edge_length <= queue_4_edge_length || queue_4_ptr_length == 0 ) && 
				( queue_1_edge_length <= queue_5_edge_length || queue_5_ptr_length == 0 ) &&
				( queue_1_edge_length <= queue_6_edge_length || queue_6_ptr_length == 0 ) &&
				( queue_1_edge_length <= queue_7_edge_length || queue_7_ptr_length == 0 ) &&
				( queue_1_edge_length <= queue_8_edge_length || queue_8_ptr_length == 0 ) && queue_1_ptr_length > 0) begin
				out_k <= queue_1_k;
				out_edge_length <= queue_1_edge_length;
				out_ptr_length <= 1;
			end
			else if(( queue_2_edge_length <= queue_3_edge_length || queue_3_ptr_length == 0 ) &&
					( queue_2_edge_length <= queue_4_edge_length || queue_4_ptr_length == 0 ) && 
					( queue_2_edge_length <= queue_5_edge_length || queue_5_ptr_length == 0 ) &&
					( queue_2_edge_length <= queue_6_edge_length || queue_6_ptr_length == 0 ) &&
					( queue_2_edge_length <= queue_7_edge_length || queue_7_ptr_length == 0 ) &&
					( queue_2_edge_length <= queue_8_edge_length || queue_8_ptr_length == 0 ) && queue_2_ptr_length > 0) begin
				out_k <= queue_2_k;
				out_edge_length <= queue_2_edge_length;
				out_ptr_length <= 1;
			end
			else if(( queue_3_edge_length <= queue_4_edge_length || queue_4_ptr_length == 0 ) && 
					( queue_3_edge_length <= queue_5_edge_length || queue_5_ptr_length == 0 ) &&
					( queue_3_edge_length <= queue_6_edge_length || queue_6_ptr_length == 0 ) &&
					( queue_3_edge_length <= queue_7_edge_length || queue_7_ptr_length == 0 ) &&
					( queue_3_edge_length <= queue_8_edge_length || queue_8_ptr_length == 0 ) && queue_3_ptr_length > 0) begin
				out_k <= queue_3_k;
				out_edge_length <= queue_3_edge_length;
				out_ptr_length <= 1;
			end
			else if(( queue_4_edge_length <= queue_5_edge_length || queue_5_ptr_length == 0 ) &&
					( queue_4_edge_length <= queue_6_edge_length || queue_6_ptr_length == 0 ) &&
					( queue_4_edge_length <= queue_7_edge_length || queue_7_ptr_length == 0 ) &&
					( queue_4_edge_length <= queue_8_edge_length || queue_8_ptr_length == 0 ) && queue_4_ptr_length > 0) begin
				out_k <= queue_4_k;
				out_edge_length <= queue_4_edge_length;
				out_ptr_length <= 1;
			end
			else if( ( queue_5_edge_length <= queue_6_edge_length || queue_6_ptr_length == 0 ) &&
				( queue_5_edge_length <= queue_7_edge_length || queue_7_ptr_length == 0 ) &&
				( queue_5_edge_length <= queue_8_edge_length || queue_8_ptr_length == 0 ) && queue_5_ptr_length > 0) begin
				out_k <= queue_5_k;
				out_edge_length <= queue_5_edge_length;
				out_ptr_length <= 1;
			end
			else if( 	( queue_6_edge_length <= queue_7_edge_length || queue_7_ptr_length == 0 ) &&
						( queue_6_edge_length <= queue_8_edge_length || queue_8_ptr_length == 0 ) && queue_6_ptr_length > 0) begin
				out_k <= queue_6_k;
				out_edge_length <= queue_6_edge_length;
				out_ptr_length <= 1;
			end
			else if( 	( queue_7_edge_length <= queue_8_edge_length || queue_8_ptr_length == 0 ) && queue_7_ptr_length > 0) begin
				out_k <= queue_7_k;
				out_edge_length <= queue_7_edge_length;
				out_ptr_length <= 1;
			end
			else if( queue_8_ptr_length > 0) begin
				out_k <= queue_8_k;
				out_edge_length <= queue_8_edge_length;
				out_ptr_length <= 1;
			end
			else begin
				out_k <= 0;
				out_edge_length <= 0;
				out_ptr_length <= 0;
			end
	end
end


endmodule




