
`default_nettype none


module length_sorter_start #(parameter K_BITS = 4,  parameter QUEUE_1_K =0, parameter QUEUE_2_K = 1, parameter SORTER_QUEUE_LENGTH_BITS = 5, parameter PTR_QUEUE_LENGTH_BITS = 5 ) (
	//---------------------- GENERAL VARIABLES---------------------------//
	input   wire                                   	clk,							// clock
    input   wire                                   	rst_n,							// Stays low as long as AFU is idle
	input wire 										um_done,						// true, if we are done processing (global)
	input wire 										um_activated,					// true, if the afu is activated
	input wire 										edge_requester_done,
	

	input wire										queue_1_edge_consumed_bit,
	input wire 										queue_2_edge_consumed_bit,

	input wire										queue_edge_added_enable,
	input wire[3:0]									queue_edge_added_num,
	input wire[K_BITS-1:0]							queue_edge_added_k,

	input wire 										ptr_add_enable,
	input wire[K_BITS-1:0]							ptr_added_k,

	input wire										ptr_consume_enable,
	input wire[K_BITS-1:0]							ptr_consumed_k,

	output reg[K_BITS-1:0]							out_k,
	output reg[9:0]									out_edge_length,
	output reg										out_ptr_length,

	output reg										k_to_request_valid_1,
	output reg										k_to_request_valid_2
);


reg [SORTER_QUEUE_LENGTH_BITS:0]	queue_1_edge_length;
reg [SORTER_QUEUE_LENGTH_BITS:0]	queue_2_edge_length;

reg [PTR_QUEUE_LENGTH_BITS:0]	queue_1_ptr_length;
reg [PTR_QUEUE_LENGTH_BITS:0]	queue_2_ptr_length;

always @(posedge clk) begin
    if (~rst_n | um_done | ~um_activated) begin
		queue_1_edge_length <= 0;
		queue_2_edge_length <= 0;

		queue_1_ptr_length <= 0;
		queue_2_ptr_length <= 0;

	end
	else begin

		// edge length 1: only consume
		if( queue_1_edge_consumed_bit && ( ~queue_edge_added_enable || ( queue_edge_added_enable && ( queue_edge_added_k != QUEUE_1_K ) ))  ) begin
			queue_1_edge_length <= queue_1_edge_length - 1;
		end
		// edge length 1: only add
		else if( (~queue_1_edge_consumed_bit &&  queue_edge_added_enable && (queue_edge_added_k == QUEUE_1_K ) ) ) begin
			queue_1_edge_length <= queue_1_edge_length + queue_edge_added_num;
		end
		// edge length 1: add and consume
		else if(queue_1_edge_consumed_bit &&  queue_edge_added_enable && (queue_edge_added_k == QUEUE_1_K ) ) begin
			queue_1_edge_length <= queue_1_edge_length -1 + queue_edge_added_num;
		end

		// edge length 2: only consume
		if( queue_2_edge_consumed_bit && ( ~queue_edge_added_enable || ( queue_edge_added_enable && ( queue_edge_added_k != QUEUE_2_K ) ))  ) begin
			queue_2_edge_length <= queue_2_edge_length - 1;
		end
		// edge length 2: only add
		else if( ~queue_2_edge_consumed_bit &&  queue_edge_added_enable && (queue_edge_added_k == QUEUE_2_K ) ) begin
			queue_2_edge_length <= queue_2_edge_length + queue_edge_added_num;
		end
		// edge length 1: add and consume
		else if(queue_2_edge_consumed_bit &&  queue_edge_added_enable && (queue_edge_added_k == QUEUE_2_K ) ) begin
			queue_2_edge_length <= queue_2_edge_length -1 + queue_edge_added_num;
		end

		// pointer length 1: only consume
		if( ( ptr_consume_enable && ptr_consumed_k == QUEUE_1_K ) && ( ~ptr_add_enable || ( ptr_add_enable && ( ptr_added_k != QUEUE_1_K ) ) ) )begin
			queue_1_ptr_length <= queue_1_ptr_length -1;
		end
		// pointer length 1: only add
		else if( ( ~ptr_consume_enable || ( ptr_consume_enable && ptr_consumed_k != QUEUE_1_K )) && ptr_add_enable && ptr_added_k == QUEUE_1_K ) begin
			queue_1_ptr_length <= queue_1_ptr_length +1;
		end
		// pointer length 1: only add + consume
		else if(ptr_consume_enable && ptr_consumed_k == QUEUE_1_K && ptr_add_enable &&  ptr_added_k == QUEUE_1_K  ) begin
			queue_1_ptr_length <= queue_1_ptr_length;
		end

		// pointer length 2: only consume
		if( ( ptr_consume_enable && ptr_consumed_k == QUEUE_2_K ) && ( ~ptr_add_enable || ( ptr_add_enable && ( ptr_added_k != QUEUE_2_K ) ) ) )begin
			queue_2_ptr_length <= queue_2_ptr_length -1;
		end
		// pointer length 2: only add
		else if( ( ~ptr_consume_enable || ( ptr_consume_enable && ptr_consumed_k != QUEUE_2_K )) && ptr_add_enable && ptr_added_k == QUEUE_2_K ) begin
			queue_2_ptr_length <= queue_2_ptr_length +1;
		end
		// pointer length 1: only add + consume
		else if(ptr_consume_enable && ptr_consumed_k == QUEUE_2_K && ptr_add_enable &&  ptr_added_k == QUEUE_2_K  ) begin
			queue_2_ptr_length <= queue_2_ptr_length;
		end

	end
end

always @(posedge clk) begin
    if (~rst_n | um_done | ~um_activated) begin
			out_k <= 0;
			out_edge_length <= 0;
			out_ptr_length <= 0;
			// out_k_valid <= 0;
	end
	else begin

			if( ( queue_1_edge_length <= queue_2_edge_length || queue_2_ptr_length == 0 ) && queue_1_ptr_length > 0) begin // if not same length, get from smaller edge queue
				out_k <= QUEUE_1_K;
				out_edge_length <= queue_1_edge_length;
				out_ptr_length <= 1;
			end
			else if( queue_2_ptr_length > 0) begin
				out_k <= QUEUE_2_K;
				out_edge_length <= queue_2_edge_length;
				out_ptr_length <= 1;
			end
			else begin
				out_k <= 0;
				out_edge_length <= 0;
				out_ptr_length <= 0;
			end
	end
end

// why 33?
// in assume we have filled to x, and we test for x < x + 1
// At 0: stage 3: issues 8 edges		
// At 1: stage 3: issues 8 edges, stage 4 sees it, creates request, increases	: 8					
// At 2: stage 3: issues 8 edges, stage 4 sees it, creates request, increases, we see that we need to stop, and set to stop		: 16 
// At 3: stage 3 stops requesting, stage 4 sees it, creates request, increases : 24
// so 33 leaves with 1 cycle padding :)
always @(posedge clk) begin
    if (~rst_n | um_done | ~um_activated) begin
		k_to_request_valid_1 <= 0;
		k_to_request_valid_2 <= 0;
	end
	else begin
		k_to_request_valid_1 <= queue_1_edge_length + 33 < 2**SORTER_QUEUE_LENGTH_BITS;
		k_to_request_valid_2 <= queue_2_edge_length + 33 < 2**SORTER_QUEUE_LENGTH_BITS;
	end
end


endmodule





`default_nettype wire



