

module mcm_bit_ack_receiver #(parameter MCM_BIT_WRITE_TAG = 11, parameter L_BITS = 3 )(
    input   wire                                   	clk,							// clock
    input   wire                                   	rst_n,							// Stays low as long as AFU is idle
	input 	wire 									um_done,						// true, if we are done processing (global)
	input 	wire 									um_activated,					// true, if the afu is activated

	output reg										mcm_bit_ack_receiver_done,			// that we are globally done
	output reg										mcm_bit_ack_receiver_epoch_done,	// that we are done with the current epoch, and change now

	input wire										mcm_bit_writer_done,				// that the mcm-bit-writer is done
	input wire										mcm_bit_writer_epoch_done,			// that the mcm-bit-writer is done for this epoch
	input wire[31:0]								mcm_bit_writer_num_writes,			// the number of writes in total

    // RX WR 
    input   wire                              		um_rx_wr_valid,	// that the tag to our write is valid
    input   wire [7:0]                       		um_rx_wr_tag,		// the tag to confirm the write we issued

	output reg[511:0]							 	debug_mcm_bit_ack_receiver
);


reg [31:0] num_acks_received;
always @(posedge clk) begin
    if (~rst_n | um_done | ~um_activated) begin
		// local vars
		num_acks_received <= 0;
	end
	else begin
		if( um_rx_wr_valid && ( um_rx_wr_tag == MCM_BIT_WRITE_TAG ) ) begin
			num_acks_received <= num_acks_received +1;
		end
	end
end


// state handling
always @(posedge clk) begin
    if (~rst_n | um_done | ~um_activated) begin
		// outgoing
		mcm_bit_ack_receiver_done <= 0;
		mcm_bit_ack_receiver_epoch_done<= 0;
	end
	else begin
		mcm_bit_ack_receiver_epoch_done <= 0;
	
		if( ~mcm_bit_ack_receiver_done ) begin

			// if the writer is done, and we have received everything, then we are done with all
			if( mcm_bit_writer_done &&  num_acks_received == mcm_bit_writer_num_writes ) begin
				mcm_bit_ack_receiver_done <= 1;
			end
			// if the writer has done its epoch, and we have received everything
			else if( mcm_bit_writer_epoch_done && num_acks_received == mcm_bit_writer_num_writes ) begin
				mcm_bit_ack_receiver_epoch_done <= 1;
			end
		end
		
	end

end

always@(posedge clk) begin
	debug_mcm_bit_ack_receiver <= {480'b0, num_acks_received };
end


endmodule





