
`define CYCLE_TIMEOUT (64'd50000)

module mwm_debug #(parameter NUM_USER_STATE_COUNTERS = 10) (
	//---------------------- GENERAL VARIABLES---------------------------//
	input   wire                                   	clk,							// clock
    input   wire                                   	rst_n,							// Stays low as long as AFU is idle
	input 	wire 									um_done,						// true, if we are done processing (global)
	input 	wire 									um_activated,					// true, if the afu is activated

	input 	wire[57:0]								debug_base_addr,

	// output reg	[NUM_USER_STATE_COUNTERS*32-1:0]	um_state_counters,
    // output reg  										um_state_counters_valid,

	input 	wire[511:0]								debug_bram_mcm_bit_receiver,	// 0
	input	wire[511:0]								debug_bram_mcm_bit_requester,	// 1
	input 	wire[511:0]								debug_edge_processor,			// 2
	input  	wire[511:0]								debug_edge_receiver,			// 3
	input 	wire[511:0]								debug_edge_requester,			// 4
	input 	wire[511:0]								debug_edge_writer,				// 5
	input 	wire[511:0]								debug_mcm_bit_ack_receiver,		// 6
	input 	wire[511:0]								debug_mcm_bit_receiver,			// 7
	input 	wire[511:0]								debug_mcm_bit_requester,		// 8
	input 	wire[511:0]								debug_mcm_bit_writer,			// 9
	input	wire[511:0]								debug_mwm_crouch_csr_ram,		// 10
	input 	wire[511:0]								debug_pointer_receiver,			// 11
	input 	wire[511:0]								debug_pointer_requester,		// 12
	input 	wire[511:0]								debug_request_interface,		// 13
	input 	wire[511:0]								debug_sorter,					// 14
	input	wire[511:0]								debug_state_controller,			// 15
	input 	wire[511:0]								debug_write_interface,			// 16

	output 	reg										debug_cl_queue_write_enable,
	output 	reg[569:0]								debug_cl_queue_buf_in,
	input 	wire									debug_cl_queue_buf_filled_next
);

reg [63:0] timeout_counter;
reg [31:0] cl_state;

/*
always @(posedge clk)
begin
	um_state_counters_valid <= 1;

	if (~rst_n | um_done | ~um_activated) begin
		um_state_counters<= 0;
	end
	else begin
		um_state_counters <= { 	timeout_counter[15:0], 			// counter to check that we have valid data
							   	debug_pointer_requester[15:0], 	// number of requests made
							   	debug_pointer_receiver[15:0], 	// number of flush pointers
								debug_pointer_receiver[47:32], 	// nuber of real pointers
								debug_edge_requester[15:0], 	// number of flush pointers removed
								debug_edge_requester[111:96], 	// number of real pointer removed
								debug_edge_requester[143:128],	// number of requests
								debug_edge_requester[175:160],	// number of edges requested
								192'b1};
	end
end
*/

always @(posedge clk)
begin
	if (~rst_n | um_done | ~um_activated) begin
		timeout_counter <= 0;
		cl_state <= 0;
		
		debug_cl_queue_write_enable <= 0;
		debug_cl_queue_buf_in <= 0;
	end
	else begin

		if( timeout_counter > `CYCLE_TIMEOUT && ~debug_cl_queue_buf_filled_next && cl_state >= 0 && cl_state <= 16) begin
			debug_cl_queue_write_enable <= 1;
			debug_cl_queue_buf_in[57:0] <= debug_base_addr + cl_state;


			if( cl_state == 0) begin
				debug_cl_queue_buf_in[569:58] <= debug_bram_mcm_bit_receiver;
			end
			else if( cl_state== 1 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_bram_mcm_bit_requester;
			end
			else if( cl_state == 2 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_edge_processor;
			end
			else if( cl_state == 3 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_edge_receiver;
			end
			else if( cl_state == 4 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_edge_requester;
			end
			else if( cl_state == 5 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_edge_writer;
			end
			else if( cl_state == 6 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_mcm_bit_ack_receiver;
			end
			else if( cl_state == 7 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_mcm_bit_receiver;
			end
			else if( cl_state == 8 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_mcm_bit_requester;
			end
			else if( cl_state == 9 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_mcm_bit_writer;
			end
			else if( cl_state == 10 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_mwm_crouch_csr_ram;
			end
			else if( cl_state == 11 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_pointer_receiver;
			end
			else if( cl_state == 12 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_pointer_requester;
			end
			else if( cl_state == 13 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_request_interface;
			end
			else if( cl_state == 14 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_sorter;
			end
			else if( cl_state == 15 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_state_controller;
			end
			else if( cl_state == 16 ) begin
				debug_cl_queue_buf_in[569:58] <= debug_write_interface;
			end

			if( cl_state == 16 ) begin
				cl_state <= 0;
				timeout_counter <= 0;
			end
			else begin
				cl_state <= cl_state  + 1 ;
			end
		end
		else begin
			debug_cl_queue_write_enable <= 0;
			debug_cl_queue_buf_in <= 0;
			timeout_counter <= timeout_counter  + 1;
		end


	end
end


endmodule





