`default_nettype none



module request_interface  #(parameter OVERFLOW_PADDING = 16, parameter NUM_REQUESTS_IN_FLIGHT = 2048, parameter BRAM_QUEUE_LENGTH_BITS= 7, parameter L_BITS = 3)  (
	//---------------------- GENERAL VARIABLES---------------------------//
	input   wire                                   	clk,							// clock
    input   wire                                   	rst_n,							// Stays low as long as AFU is idle
	input wire 										um_done,						// true, if we are done processing (global)
	input wire 										um_activated,					// true, if the afu is activated


	input wire[57:0]								ptr_base_addr,
	input wire[57:0]								edges_base_addr,
	input wire[57:0]								out_mcm_bits_base_addr,
	input wire[31:0]								num_ptrs,
	input wire[31:0]								num_edges,
	
    //----------------------ADDRESS VARIABLES ---------------------------//
    output  reg  [57:0]                            um_tx_rd_addr,	// read req. address (accesses 512bits)
    output  reg  [7:0]                             um_tx_rd_tag,	// read req. tag
    output  reg                                    um_tx_rd_valid,	// indicated that tag and addr. are valid
    input   wire                                   um_tx_rd_ready,	// framework tells that we can issue request
	input   wire                                   um_rx_rd_valid,

	//---------------------- POINTER REQUESTER ---------------------------//
	output reg 										pointer_requester_queue_buf_filled_next,
	input wire [65:0]								pointer_requester_queue_buf_in,
	input wire										pointer_requester_queue_write_enable,
	input wire[31:0]								pointer_receiver_num_resp_received,
	input wire[31:0]								pointer_receiver_space_left,

	//---------------------- EDGE REQUESTER ---------------------------//
	output reg 										edge_requester_queue_buf_filled_next,
	input wire [65:0]								edge_requester_queue_buf_in,
	input wire 										edge_requester_queue_write_enable,
	input wire[31:0]								edge_receiver_num_resp_received,
	input wire[31:0]								edge_receiver_space_left,

	//---------------------- MCM-BIT REQUESTER FOR EDGE PROCESSOR ---------------------------//
	output reg 										mcm_bit_requester_queue_buf_filled_next,
	input wire [65:0]								mcm_bit_requester_queue_buf_in,
	input wire 										mcm_bit_requester_queue_write_enable,
	input wire[31:0]								mcm_bit_receiver_num_resp_received,
	input wire[31:0]								mcm_bit_receiver_space_left,

    //---------------------- MCM-BIT REQUESTER FOR EDGE BRAM ---------------------------//
	output reg										bram_mcm_bit_requester_queue_buf_filled_next,
	input wire [65:0]								bram_mcm_bit_requester_queue_buf_in,
	input wire 										bram_mcm_bit_requester_queue_write_enable,
	input wire[31:0]								bram_mcm_bit_receiver_num_resp_received,
	input wire[31:0]								bram_mcm_bit_receiver_space_left,

	output reg[511:0]								debug_request_interface
);




reg									mcm_m_axis_ready;
wire								mcm_m_axis_valid;
wire [BRAM_QUEUE_LENGTH_BITS-1:0]	mcm_data_count;
wire [65:0]							mcm_m_axis_data;
reg  [65:0]							mcm_data_buffer;
reg									mcm_data_buffer_valid;


nlb_axififo #(.BUS_SIZE_ADDR(BRAM_QUEUE_LENGTH_BITS), .BUS_SIZE_DATA(66), .PROG_FULL_THRESH(0 ) ) mcm_axififo	(
    .clk (clk),
    .rst(~rst_n),
    .s_axis_valid(mcm_bit_requester_queue_write_enable),
    .s_axis_ready(),
    .s_axis_data(mcm_bit_requester_queue_buf_in),
    .s_axis_last(1'b0),
    .m_axis_valid(mcm_m_axis_valid),
    .m_axis_ready(mcm_m_axis_ready),
    .m_axis_data(mcm_m_axis_data),
    .m_axis_last(),
    .data_count(mcm_data_count)
);



reg									bram_mcm_bit_m_axis_ready;
wire								bram_mcm_bit_m_axis_valid;
wire [BRAM_QUEUE_LENGTH_BITS-1:0]	bram_mcm_bit_data_count;
wire [65:0]							bram_mcm_bit_m_axis_data;
reg  [65:0]							bram_mcm_bit_data_buffer;
reg									bram_mcm_bit_data_buffer_valid;

nlb_axififo #(.BUS_SIZE_ADDR(BRAM_QUEUE_LENGTH_BITS), .BUS_SIZE_DATA(66), .PROG_FULL_THRESH(0 ) ) bram_mcm_bit_axififo	(
    .clk (clk),
    .rst(~rst_n),
    .s_axis_valid(bram_mcm_bit_requester_queue_write_enable),
    .s_axis_ready(),
    .s_axis_data(bram_mcm_bit_requester_queue_buf_in),
    .s_axis_last(1'b0),
    .m_axis_valid(bram_mcm_bit_m_axis_valid),
    .m_axis_ready(bram_mcm_bit_m_axis_ready),
    .m_axis_data(bram_mcm_bit_m_axis_data),
    .m_axis_last(),
    .data_count(bram_mcm_bit_data_count)
);



reg									edge_m_axis_ready;
wire								edge_m_axis_valid;
wire [BRAM_QUEUE_LENGTH_BITS-1:0]	edge_data_count;
wire [65:0]							edge_m_axis_data;
reg  [65:0]							edge_data_buffer;
reg									edge_data_buffer_valid;

nlb_axififo #(.BUS_SIZE_ADDR(BRAM_QUEUE_LENGTH_BITS), .BUS_SIZE_DATA(66), .PROG_FULL_THRESH(0 ) ) edge_axififo	(
    .clk (clk),
    .rst(~rst_n),
    .s_axis_valid(edge_requester_queue_write_enable),
    .s_axis_ready(),
    .s_axis_data(edge_requester_queue_buf_in),
    .s_axis_last(1'b0),
    .m_axis_valid(edge_m_axis_valid),
    .m_axis_ready(edge_m_axis_ready),
    .m_axis_data(edge_m_axis_data),
    .m_axis_last(),
    .data_count(edge_data_count)
);


reg									ptr_m_axis_ready;
wire								ptr_m_axis_valid;
wire [BRAM_QUEUE_LENGTH_BITS-1:0]	ptr_data_count;
wire [65:0]							ptr_m_axis_data;
reg  [65:0]							ptr_data_buffer;
reg									ptr_data_buffer_valid;

nlb_axififo #(.BUS_SIZE_ADDR(BRAM_QUEUE_LENGTH_BITS), .BUS_SIZE_DATA(66), .PROG_FULL_THRESH(0 ) ) pointer_axififo	(
    .clk (clk),
    .rst(~rst_n),
    .s_axis_valid(pointer_requester_queue_write_enable),
    .s_axis_ready(),
    .s_axis_data(pointer_requester_queue_buf_in),
    .s_axis_last(1'b0),
    .m_axis_valid(ptr_m_axis_valid),
    .m_axis_ready(ptr_m_axis_ready),
    .m_axis_data(ptr_m_axis_data),
    .m_axis_last(),
    .data_count(ptr_data_count)
);



reg [31:0] edge_requester_num_issued_requests;
reg [31:0] pointer_requester_num_issued_requests;
reg [31:0] mcm_bit_requester_num_issued_requests;
reg [31:0] bram_mcm_bit_requester_num_issued_requests;

reg [31:0] num_issued_requests;		// count the number of issued read requests
reg [31:0] num_received_responses;	// count the number of received read responses

reg [31:0] edge_ntotal_read;
reg [31:0] edge_ntotal_write;

reg [31:0] ptr_ntotal_read;
reg [31:0] ptr_ntotal_write;

reg [31:0] mcm_ntotal_read;
reg [31:0] mcm_ntotal_write;

reg [31:0] bram_mcm_bit_ntotal_read;
reg [31:0] bram_mcm_bit_ntotal_write;

//---------------------- Handle and track writes to the queue ---------------------------//
always @(posedge clk)
begin
	if (~rst_n | um_done | ~um_activated) begin
		ptr_ntotal_write <= 0;
		pointer_requester_queue_buf_filled_next <= 0;

		edge_ntotal_write <= 0;
		edge_requester_queue_buf_filled_next <= 0;

		mcm_ntotal_write <= 0;
		mcm_bit_requester_queue_buf_filled_next <= 0;

		bram_mcm_bit_ntotal_write <= 0;
		bram_mcm_bit_requester_queue_buf_filled_next<= 0;
	end
	else begin
		// mcm bram bit requester
		if( bram_mcm_bit_requester_queue_write_enable ) begin
			bram_mcm_bit_ntotal_write <= bram_mcm_bit_ntotal_write +1;
		end
		bram_mcm_bit_requester_queue_buf_filled_next <= ( bram_mcm_bit_ntotal_write  - bram_mcm_bit_ntotal_read + OVERFLOW_PADDING  )  >= 2**BRAM_QUEUE_LENGTH_BITS ;


		// mcm bit requester
		if( mcm_bit_requester_queue_write_enable ) begin
			mcm_ntotal_write <= mcm_ntotal_write +1;
		end
		mcm_bit_requester_queue_buf_filled_next <= ( mcm_ntotal_write  - mcm_ntotal_read + OVERFLOW_PADDING  )  >= 2**BRAM_QUEUE_LENGTH_BITS ;

		// pointer requester
		if(pointer_requester_queue_write_enable  ) begin
			ptr_ntotal_write <= ptr_ntotal_write +1;
		end
		pointer_requester_queue_buf_filled_next <= ( ptr_data_count+ OVERFLOW_PADDING ) >= 2**BRAM_QUEUE_LENGTH_BITS;
		

		// edge requester
		if( edge_requester_queue_write_enable ) begin
			edge_ntotal_write <= edge_ntotal_write +1;
		end
		edge_requester_queue_buf_filled_next <= ( edge_data_count + OVERFLOW_PADDING) >= 2**BRAM_QUEUE_LENGTH_BITS;


	end
end

reg [5:0]	debug_case;

reg			hold_request_valid;
reg [57:0]	hold_request_address;
reg [7:0]	hold_request_tag;

//---------------------- GENERATE READ REQUESTS ---------------------------//
always @(posedge clk) begin
    if (~rst_n | um_done | ~um_activated) begin
			
		// local variables
        edge_requester_num_issued_requests <= 0;
		pointer_requester_num_issued_requests <= 0;
		mcm_bit_requester_num_issued_requests <= 0;
		bram_mcm_bit_requester_num_issued_requests <= 0;
		
		// queue tracking for debug
		ptr_ntotal_read <= 0;
		edge_ntotal_read <= 0;
		mcm_ntotal_read <= 0;
		bram_mcm_bit_ntotal_read <= 0;

		// output variables for reading
		um_tx_rd_addr <= 0;
		um_tx_rd_tag <= 0;	
		um_tx_rd_valid <= 0;

		// queue handling
		
		ptr_data_buffer_valid <= 0;
		edge_data_buffer_valid <= 0;
		mcm_data_buffer_valid <= 0;
		bram_mcm_bit_data_buffer_valid <= 0;
		hold_request_valid <= 0;

		// drain queues
		if( (um_done || ~um_activated) && rst_n ) begin
			bram_mcm_bit_m_axis_ready <= 1;
			mcm_m_axis_ready <= 1;
			edge_m_axis_ready <= 1;
			ptr_m_axis_ready<= 1;
		end
		else begin
			bram_mcm_bit_m_axis_ready <= 0;
			mcm_m_axis_ready <= 0;
			edge_m_axis_ready <= 0;
			ptr_m_axis_ready<= 0;
		end


		// debug
		debug_case <= 0;

	end
	else begin
		um_tx_rd_valid <= 0;
		debug_case <= 0;

		// if we have a pending request, which did not go through, try again
		if( hold_request_valid && um_tx_rd_ready ) begin
			hold_request_valid <= 0;
			um_tx_rd_addr <= hold_request_address;
			um_tx_rd_tag <= hold_request_tag;
			um_tx_rd_valid <= 1;
		end

		// a request did not go through
		if( !um_tx_rd_ready && um_tx_rd_valid ) begin
			hold_request_valid <= 1;
			hold_request_address <= um_tx_rd_addr;
			hold_request_tag <= um_tx_rd_tag;
		end

		// mcm-bit requester can request:
		if( !( hold_request_valid && um_tx_rd_ready) &&
			!(!um_tx_rd_ready && um_tx_rd_valid ) &&
			( um_tx_rd_ready && ( num_issued_requests - num_received_responses < NUM_REQUESTS_IN_FLIGHT) ) &&
			( mcm_bit_requester_num_issued_requests + OVERFLOW_PADDING <  mcm_bit_receiver_space_left +  mcm_bit_receiver_num_resp_received ) &&
			( mcm_data_buffer_valid || ( mcm_m_axis_valid && mcm_m_axis_ready ) ) 
			) begin

				debug_case <= 1;

				
				if( mcm_data_buffer_valid ) begin
					um_tx_rd_addr <= mcm_data_buffer[65:8];
					um_tx_rd_tag <= mcm_data_buffer[7:0];
					um_tx_rd_valid <= 1;
					mcm_m_axis_ready<= 1;
					mcm_bit_requester_num_issued_requests <= mcm_bit_requester_num_issued_requests + 1;
		
					// sub-case: if we have valid data on the wire, move to the buffer
					if( mcm_m_axis_valid && mcm_m_axis_ready) begin
						mcm_data_buffer <= mcm_m_axis_data;
						mcm_data_buffer_valid <= 1;
						mcm_ntotal_read <= mcm_ntotal_read +1;
					end

					// sub-case: we do not have valid data on the wire, set buffer to invalid
					if( !(mcm_m_axis_valid && mcm_m_axis_ready) ) begin
						mcm_data_buffer_valid <= 0;
					end
				end
				else begin
					um_tx_rd_addr <= mcm_m_axis_data[65:8];
					um_tx_rd_tag <= mcm_m_axis_data[7:0];
					um_tx_rd_valid <= 1;
					mcm_data_buffer_valid <= 0;
					mcm_m_axis_ready<= 1;
					mcm_ntotal_read <= mcm_ntotal_read + 1;
					mcm_bit_requester_num_issued_requests <= mcm_bit_requester_num_issued_requests + 1;
				end
		end
		else begin

			// edge requester can not request
			
			// handling the edge requester:
			if( mcm_m_axis_valid && mcm_m_axis_ready ) begin
				mcm_data_buffer <= mcm_m_axis_data;
				mcm_data_buffer_valid <= 1;
				mcm_m_axis_ready <= 0;
				mcm_ntotal_read <= mcm_ntotal_read + 1;
			end
			else if( !mcm_data_buffer_valid && !mcm_m_axis_ready ) begin
				mcm_m_axis_ready <= 1;
			end
		end


		// bram mcm-bit requester can request
		if( !( hold_request_valid && um_tx_rd_ready) &&
			!(!um_tx_rd_ready && um_tx_rd_valid ) &&
			!( ( um_tx_rd_ready && ( num_issued_requests - num_received_responses < NUM_REQUESTS_IN_FLIGHT) ) &&
			( mcm_bit_requester_num_issued_requests + OVERFLOW_PADDING <  mcm_bit_receiver_space_left +  mcm_bit_receiver_num_resp_received ) &&
			( mcm_data_buffer_valid || ( mcm_m_axis_valid && mcm_m_axis_ready ) ) 
			)
			&&
			( um_tx_rd_ready && ( num_issued_requests - num_received_responses < NUM_REQUESTS_IN_FLIGHT) ) &&
			( bram_mcm_bit_requester_num_issued_requests + OVERFLOW_PADDING <  bram_mcm_bit_receiver_space_left +  bram_mcm_bit_receiver_num_resp_received ) &&
			( bram_mcm_bit_data_buffer_valid || ( bram_mcm_bit_m_axis_valid && bram_mcm_bit_m_axis_ready ) )
		) begin
				
				debug_case <= 3;
				if( bram_mcm_bit_data_buffer_valid ) begin
					um_tx_rd_addr <= bram_mcm_bit_data_buffer[65:8];
					um_tx_rd_tag <= bram_mcm_bit_data_buffer[7:0];
					um_tx_rd_valid <= 1;
					bram_mcm_bit_m_axis_ready<= 1;
					bram_mcm_bit_requester_num_issued_requests <= bram_mcm_bit_requester_num_issued_requests + 1;
		
					// sub-case: if we have valid data on the wire, move to the buffer
					if( bram_mcm_bit_m_axis_valid && bram_mcm_bit_m_axis_ready) begin
						bram_mcm_bit_data_buffer <= bram_mcm_bit_m_axis_data;
						bram_mcm_bit_data_buffer_valid <= 1;
						bram_mcm_bit_ntotal_read <= bram_mcm_bit_ntotal_read +1;
					end

					// sub-case: we do not have valid data on the wire, set buffer to invalid
					if( !(bram_mcm_bit_m_axis_valid && bram_mcm_bit_m_axis_ready) ) begin
						bram_mcm_bit_data_buffer_valid <= 0;
					end
				end
				else begin
					um_tx_rd_addr <= bram_mcm_bit_m_axis_data[65:8];
					um_tx_rd_tag <= bram_mcm_bit_m_axis_data[7:0];
					um_tx_rd_valid <= 1;
					bram_mcm_bit_data_buffer_valid <= 0;
					bram_mcm_bit_m_axis_ready<= 1;
					bram_mcm_bit_ntotal_read <= bram_mcm_bit_ntotal_read + 1;
					bram_mcm_bit_requester_num_issued_requests <= bram_mcm_bit_requester_num_issued_requests + 1;
				end
		
		end


		else begin

				// handling the bram_mcm_bit requester:
				if( bram_mcm_bit_m_axis_valid && bram_mcm_bit_m_axis_ready ) begin
					bram_mcm_bit_data_buffer <= bram_mcm_bit_m_axis_data;
					bram_mcm_bit_data_buffer_valid <= 1;
					bram_mcm_bit_m_axis_ready <= 0;
					bram_mcm_bit_ntotal_read <= bram_mcm_bit_ntotal_read + 1;
				end
				else if( !bram_mcm_bit_data_buffer_valid && !bram_mcm_bit_m_axis_ready ) begin
					bram_mcm_bit_m_axis_ready <= 1;
				end
		end 


		// edge requester can request:
		if(	!( hold_request_valid && um_tx_rd_ready) &&
			!(!um_tx_rd_ready && um_tx_rd_valid ) &&
			!( ( um_tx_rd_ready && ( num_issued_requests - num_received_responses < NUM_REQUESTS_IN_FLIGHT) ) &&
			( mcm_bit_requester_num_issued_requests + OVERFLOW_PADDING <  mcm_bit_receiver_space_left +  mcm_bit_receiver_num_resp_received ) &&
			( mcm_data_buffer_valid || ( mcm_m_axis_valid && mcm_m_axis_ready ) ) 
			) 
			&&
			!( ( um_tx_rd_ready && ( num_issued_requests - num_received_responses < NUM_REQUESTS_IN_FLIGHT) ) &&
			( bram_mcm_bit_requester_num_issued_requests + OVERFLOW_PADDING <  bram_mcm_bit_receiver_space_left +  bram_mcm_bit_receiver_num_resp_received ) &&
			( bram_mcm_bit_data_buffer_valid || ( bram_mcm_bit_m_axis_valid && bram_mcm_bit_m_axis_ready ) )
			)
			&&
				( um_tx_rd_ready && ( num_issued_requests - num_received_responses < NUM_REQUESTS_IN_FLIGHT) ) &&
				( edge_requester_num_issued_requests + OVERFLOW_PADDING  <  edge_receiver_space_left +  edge_receiver_num_resp_received ) &&
				( edge_data_buffer_valid ||  (edge_m_axis_valid && edge_m_axis_ready) ) 
		  ) begin

				debug_case <= 5;

				if( edge_data_buffer_valid ) begin
					um_tx_rd_addr <= edge_data_buffer[65:8];
					um_tx_rd_tag <= edge_data_buffer[7:0];
					um_tx_rd_valid <= 1;
					edge_m_axis_ready<= 1;
					edge_requester_num_issued_requests <= edge_requester_num_issued_requests + 1;
		
					// sub-case: if we have valid data on the wire, move to the buffer
					if( edge_m_axis_valid && edge_m_axis_ready) begin
						edge_data_buffer <= edge_m_axis_data;
						edge_data_buffer_valid <= 1;
						edge_ntotal_read <= edge_ntotal_read +1;
					end

					// sub-case: we do not have valid data on the wire, set buffer to invalid
					if( !(edge_m_axis_valid && edge_m_axis_ready) ) begin
						edge_data_buffer_valid <= 0;
					end
				end
				else begin
					um_tx_rd_addr <= edge_m_axis_data[65:8];
					um_tx_rd_tag <= edge_m_axis_data[7:0];
					um_tx_rd_valid <= 1;
					edge_data_buffer_valid <= 0;
					edge_m_axis_ready<= 1;
					edge_ntotal_read <= edge_ntotal_read + 1;
					edge_requester_num_issued_requests <= edge_requester_num_issued_requests + 1;
				end
		end
		else begin

			// edge requester can not request
			
			// handling the edge requester:
			if( edge_m_axis_valid && edge_m_axis_ready ) begin
				edge_data_buffer <= edge_m_axis_data;
				edge_data_buffer_valid <= 1;
				edge_m_axis_ready <= 0;
				edge_ntotal_read <= edge_ntotal_read + 1;
			end
			else if( !edge_data_buffer_valid && !edge_m_axis_ready ) begin
				edge_m_axis_ready <= 1;
			end
		end


		// ptr-requester can request:
		if(	!( hold_request_valid && um_tx_rd_ready) &&
			!(!um_tx_rd_ready && um_tx_rd_valid ) &&
			!( ( um_tx_rd_ready && ( num_issued_requests - num_received_responses < NUM_REQUESTS_IN_FLIGHT) ) &&
			( mcm_bit_requester_num_issued_requests + OVERFLOW_PADDING <  mcm_bit_receiver_space_left +  mcm_bit_receiver_num_resp_received ) &&
			( mcm_data_buffer_valid || ( mcm_m_axis_valid && mcm_m_axis_ready ) ) 
			) 
			&&
			!( ( um_tx_rd_ready && ( num_issued_requests - num_received_responses < NUM_REQUESTS_IN_FLIGHT) ) &&
			( bram_mcm_bit_requester_num_issued_requests + OVERFLOW_PADDING <  bram_mcm_bit_receiver_space_left +  bram_mcm_bit_receiver_num_resp_received ) &&
			( bram_mcm_bit_data_buffer_valid || ( bram_mcm_bit_m_axis_valid && bram_mcm_bit_m_axis_ready ))
			)
			&&
		 	!(	( um_tx_rd_ready & ( num_issued_requests - num_received_responses < NUM_REQUESTS_IN_FLIGHT) ) &&
				( edge_requester_num_issued_requests + OVERFLOW_PADDING  <  edge_receiver_space_left +  edge_receiver_num_resp_received ) &&
				( edge_data_buffer_valid ||  (edge_m_axis_valid && edge_m_axis_ready) ) )
			&& 
				( um_tx_rd_ready & ( num_issued_requests - num_received_responses < NUM_REQUESTS_IN_FLIGHT) ) &&
				( pointer_requester_num_issued_requests + OVERFLOW_PADDING < pointer_receiver_space_left + pointer_receiver_num_resp_received ) &&
				( ptr_data_buffer_valid || (ptr_m_axis_valid && ptr_m_axis_ready ) )
			) begin


			debug_case<= 7;

			if( ptr_data_buffer_valid ) begin
				um_tx_rd_addr <= ptr_data_buffer[65:8];
				um_tx_rd_tag <= ptr_data_buffer[7:0];
				um_tx_rd_valid <= 1;
				ptr_m_axis_ready<= 1;
				pointer_requester_num_issued_requests <= pointer_requester_num_issued_requests + 1;
		
				// sub-case: if we have valid data on the wire, move to the buffer
				if( ptr_m_axis_valid && ptr_m_axis_ready) begin
					ptr_data_buffer <= ptr_m_axis_data;
					ptr_data_buffer_valid <= 1;
					ptr_ntotal_read <= ptr_ntotal_read +1;
				end

				// sub-case: we do not have valid data on the wire, set buffer to invalid
				if( !(ptr_m_axis_valid && ptr_m_axis_ready) ) begin
					ptr_data_buffer_valid <= 0;
				end
			end
			else begin
				um_tx_rd_addr <= ptr_m_axis_data[65:8];
				um_tx_rd_tag <= ptr_m_axis_data[7:0];
				um_tx_rd_valid <= 1;
				ptr_data_buffer_valid <= 0;
				ptr_m_axis_ready<= 1;
				ptr_ntotal_read <= ptr_ntotal_read + 1;
				pointer_requester_num_issued_requests<= pointer_requester_num_issued_requests  + 1;
			end
		end
		else begin
			if( ptr_m_axis_valid && ptr_m_axis_ready ) begin
				ptr_data_buffer <= ptr_m_axis_data;
				ptr_data_buffer_valid <= 1;
				ptr_m_axis_ready <= 0;
				ptr_ntotal_read <= ptr_ntotal_read + 1;
			end
			else if( !ptr_data_buffer_valid && !ptr_m_axis_valid  ) begin
				ptr_m_axis_ready <= 1;
			end
		end


	end
end



reg [31:0]		debug_address_error32;
reg	[31:0]		debug_case_error;
		
always @(posedge clk) begin
	 if (~rst_n | um_done | ~um_activated) begin
		debug_address_error32 <= 0;
		debug_case_error<= 0;
	end
	/*
	// skipped, due to timing constraints: seems to influence the result
	else if( um_tx_rd_valid ) begin
			if( um_tx_rd_addr >= ptr_base_addr && um_tx_rd_addr <= (ptr_base_addr + (num_ptrs*7 >> 5) + 1) ) begin // similar to: num_ptrs/ 5  = num_ptrs *7 / 32 = num_ptrs * 0.21875
				// then ok
			end
			else if( um_tx_rd_addr >= edges_base_addr && um_tx_rd_addr <= (edges_base_addr + (num_edges >> 3) ) ) begin
				// then ok
			end
			else if( um_tx_rd_addr >= out_mcm_bits_base_addr && um_tx_rd_addr<= ( out_mcm_bits_base_addr + ( (num_ptrs << 9) >> L_BITS) + 1 ) ) begin
				// then ok
			end
			else begin
				debug_address_error32 <= 1;
				debug_case_error <= debug_case;
			end
	end
	*/
end





always @(posedge clk) begin
	 if (~rst_n | um_done | ~um_activated) begin
		num_issued_requests <= 0;
	end
	else if( um_tx_rd_valid && um_tx_rd_ready) begin
		num_issued_requests <= num_issued_requests  + 1;
	end
end

always @(posedge clk) begin
    if (~rst_n | um_done | ~um_activated) begin
			num_received_responses <= 0;
	end
	else begin
		if( um_rx_rd_valid ) begin
			num_received_responses <= num_received_responses +1;
		end
	end
end


/*
// high, if we do not output data, but we could
reg				do_not_work_valid;
reg [31:0]		do_not_work_cycle_counter;
// high, if we hava data, but cannot output data
reg				can_not_output_valid;
reg [31:0]		can_not_output_cycle_counter;
// high, if we do not have data, but could output
reg				can_not_input_valid;
reg [31:0]		can_not_input_cycle_counter;


always @(posedge clk) begin
	if (~rst_n | um_done | ~um_activated) begin
		do_not_work_valid <= 0;
		do_not_work_cycle_counter <= 0;
		can_not_output_valid <= 0;
		can_not_output_cycle_counter <= 0;
		can_not_input_valid <= 0;
		can_not_input_cycle_counter <= 0;
	end
	else begin
		do_not_work_valid <= 0;
		can_not_output_valid <= 0;
		can_not_input_valid <= 0;

		if( ~um_tx_rd_valid && ( mcm_data_count > 0 || bram_mcm_bit_data_count > 0 || edge_data_count > 0 || ptr_data_count > 0) ) begin
			do_not_work_valid<= 1;
			do_not_work_cycle_counter <= do_not_work_cycle_counter + 1;
		end

		if(  ( mcm_data_count > 0 || bram_mcm_bit_data_count > 0 || edge_data_count > 0 || ptr_data_count > 0) && ~um_tx_rd_ready) begin
			can_not_output_valid <= 1;
			can_not_output_cycle_counter <= can_not_output_cycle_counter  + 1;
		end

		if(( mcm_data_count == 0 && bram_mcm_bit_data_count == 0 && edge_data_count == 0 && ptr_data_count == 0) && um_tx_rd_ready ) begin
			can_not_input_valid <= 1;
			can_not_input_cycle_counter <= can_not_input_cycle_counter + 1;
		end
	end
end
*/


always @(posedge clk) begin
	debug_request_interface <= {128'b0,
								num_issued_requests, num_received_responses,
								debug_case_error, debug_address_error32,
								edge_ntotal_read, edge_ntotal_write, 
								ptr_ntotal_read, ptr_ntotal_write,
								mcm_ntotal_read, mcm_ntotal_write,
								bram_mcm_bit_ntotal_read, bram_mcm_bit_ntotal_write };
end

endmodule






`default_nettype wire




