`default_nettype none

module sorter #(parameter K_BITS = 2, parameter K_BIT_2 = 2, parameter SORTER_FIRST_QUEUE_LENGTH_BITS= 7, parameter SORTER_INNER_QUEUE_LENGTH_BITS = 7, parameter BRAM_OVERFLOW_PADDING = 8 ) (
	//---------------------- GENERAL VARIABLES---------------------------//
	input   wire                                   	clk,							// clock
    input   wire                                   	rst_n,							// Stays low as long as AFU is idle
	input 	wire 									um_done,						// true, if we are done processing (global)
	input 	wire 									um_activated,					// true, if the afu is activated
	output reg 										sorter_done,
		
	//---------------------- VARIABLES FOR THE K MANY EDGE QUEUES FROM THE EDGE RECEIVER ---------------------------//
	input wire[K_BITS-1:0]						in_queue_write_enable_k,
	input wire									in_queue_write_enable,					
	input wire[95:0] 							in_queue_buf_in,						// (epoch, u, v, w)
	// output wire									in_queue_filled_next[0:2**K_BITS-1],

	input wire 									edge_receiver_done,

	//---------------------- VARIABLES FOR THE OUTPUT QUEUE GOING TO THE MCM-BIT REQUESTER ---------------------------//
	output reg									out_queue_write_enable,
	output reg[127:0]							out_queue_buf_in,
	input wire									out_queue_buf_filled_next,

	input wire[57:0]							edge_debug_address,
	output reg									out_edge_debug_write_enable,
	output reg[569:0]							out_edge_debug_buf_in,
	input wire									out_edge_debug_buf_filled_next,
	output wire									sorter_edge_consumed[0:2**K_BITS-1],

	output reg[511:0]							debug_sorter
);

wire[31:0] all_ones_32_bit;
assign all_ones_32_bit = ~0;

reg out_queue_write_enable_intermediate;
reg[95:0] out_queue_buf_in_intermediate;

reg [31:0] debug_n_edges_in_normal;
reg [31:0] debug_n_edges_in_flush;

reg		in_queue_write_enable_preprocess;
reg [95:0]	in_queue_buf_in_preprocess;
reg	[K_BITS-1:0]	in_queue_write_enable_k_preprocess;
always @(posedge clk) begin
	if (~rst_n | um_done | ~um_activated) begin
		debug_n_edges_in_normal <= 0;
		debug_n_edges_in_flush <= 0;
		in_queue_write_enable_preprocess <= 0;
		in_queue_write_enable_k_preprocess <= 0;
	end
	else begin
		if( in_queue_write_enable ) begin
			if( all_ones_32_bit != in_queue_buf_in[63:32] ) begin
				debug_n_edges_in_normal <= debug_n_edges_in_normal  + 1;
			end
			else begin
				debug_n_edges_in_flush <= debug_n_edges_in_flush +1;
			end
		end

		if( in_queue_write_enable ) begin
			in_queue_write_enable_preprocess <= 1;
			in_queue_buf_in_preprocess <= in_queue_buf_in;
			in_queue_buf_in_preprocess[95:64] <= in_queue_buf_in[95:64] -1;
			in_queue_write_enable_k_preprocess <= in_queue_write_enable_k;
		end
		else begin
			in_queue_write_enable_preprocess <= 0;
		end
		
	end
end

reg [31:0] debug_n_edges_out_normal;
reg [31:0] debug_n_edges_out_flush;
always @(posedge clk) begin
	if (~rst_n | um_done | ~um_activated) begin
		debug_n_edges_out_normal <= 0;
		debug_n_edges_out_flush <= 0;
	end
	else begin
		if( out_queue_write_enable_intermediate ) begin
			if( all_ones_32_bit != out_queue_buf_in_intermediate[63:32] ) begin
				debug_n_edges_out_normal <= debug_n_edges_out_normal  + 1;
			end
			else begin
				debug_n_edges_out_flush <= debug_n_edges_out_flush + 1;
			end
		end
	end
end


reg[31:0] edge_debug_offset;
reg[31:0] edge_debug_num_cl;
reg[512-1:0] edge_debug_cl;

// sort out the "flush" edges and write the edges out to the host for debug reason
always @(posedge clk) begin
	if (~rst_n | um_done | ~um_activated) begin
		out_queue_write_enable <= 0;
		out_queue_buf_in<= 0;
		

		edge_debug_offset <= 0;
		edge_debug_num_cl <= 0;
		out_edge_debug_write_enable <= 0;
		edge_debug_cl <= 0;
	end
	else begin
		out_queue_write_enable <= 0;
		out_edge_debug_write_enable <= 0;

		if( out_queue_write_enable_intermediate ) begin
			if( all_ones_32_bit != out_queue_buf_in_intermediate[63:32] ) begin
				out_queue_write_enable <= 1;
				out_queue_buf_in[63:0] <= out_queue_buf_in_intermediate;
				out_queue_buf_in[95:64] <= out_queue_buf_in_intermediate[95:64] + 1;
				out_queue_buf_in[127:96] <= out_queue_buf_in_intermediate[95:64+K_BITS];

				if( edge_debug_offset +1 == 4 ) begin
					out_edge_debug_write_enable <= 1;
					out_edge_debug_buf_in[57:0] <=  edge_debug_address + edge_debug_num_cl;
					out_edge_debug_buf_in[441:58] <= edge_debug_cl[511:128];
					out_edge_debug_buf_in[505:442] <= out_queue_buf_in_intermediate;
					out_edge_debug_buf_in[537:506] <= out_queue_buf_in_intermediate[95:64] + 1;
					out_edge_debug_buf_in[569:538] <= out_queue_buf_in_intermediate[95:64+K_BITS];
					
					edge_debug_num_cl <= edge_debug_num_cl  + 1;
					edge_debug_offset <= 0;
					edge_debug_cl <= 0;
				end	
				else begin
					edge_debug_cl[447:384] <=  out_queue_buf_in_intermediate;
					edge_debug_cl[479:448] <= out_queue_buf_in_intermediate[95:64] +1;
					edge_debug_cl[511:480] <= out_queue_buf_in_intermediate[95:64+K_BITS];
					edge_debug_cl[383:0] <= edge_debug_cl[511:128];
					edge_debug_offset <= edge_debug_offset  + 1;
				end
			end
		end
		else if (  sorter_done && (edge_debug_offset > 0) ) begin
			out_edge_debug_write_enable <= 1;
			out_edge_debug_buf_in <= {edge_debug_cl, edge_debug_address + edge_debug_num_cl };
					
			edge_debug_num_cl <= edge_debug_num_cl  + 1;
			edge_debug_offset <= 0;
			edge_debug_cl <= 0;
		end
	end
end

wire[15:0] queue_length[0:2**K_BIT_2-1];

generate
genvar i,j;

begin
	wire  connect_out_queue_write_enable[0:K_BIT_2-1][0:2**K_BIT_2-1]; //K_BIT is the number of levels
	wire[95:0] connect_out_queue_in  	[0:K_BIT_2-1][0:2**K_BIT_2-1];
	wire connect_out_queue_filled_next  [0:K_BIT_2-1][0:2**K_BIT_2-1];
	wire connect_local_sorter_done 		[0:K_BIT_2-1][0:2**K_BIT_2-1];
	

	// for all levels
	for(j = 0; j < K_BIT_2; j++) begin : sorter_outer_for
		// for all sorter-nodes in this level
		for(i = 0; i < (2**(K_BIT_2 -1 -j)); i++) begin : sorter_inner_for
	
			if( j == 0) begin
				// special case: the first row (input)
				sorter_node_start #(.BRAM_QUEUE_LENGTH_BITS(SORTER_FIRST_QUEUE_LENGTH_BITS), .OVERFLOW_PADDING(BRAM_OVERFLOW_PADDING), .K_BITS(K_BITS) ) sorter_node_start (
					.clk						(clk),
   					.rst_n						(rst_n),
					.um_done					(um_done),
					.um_activated				(um_activated),
					.sorter_done				(sorter_done),
					.edge_receiver_done			(edge_receiver_done),
					.local_sorter_done			(connect_local_sorter_done[j][i]),

					.in_queue_write_enable_0	(in_queue_write_enable_preprocess && (in_queue_write_enable_k_preprocess == 2*i) ),
					.in_queue_buf_in_0			(in_queue_buf_in_preprocess),
					.sorter_edge_consumed_0		(sorter_edge_consumed[2*i]),
					
					.in_queue_write_enable_1	(in_queue_write_enable_preprocess && (in_queue_write_enable_k_preprocess == 2*i+1) ),
					.in_queue_buf_in_1			(in_queue_buf_in_preprocess),
					.sorter_edge_consumed_1		(sorter_edge_consumed[2*i+1]),

					.out_queue_write_enable		(connect_out_queue_write_enable[j][i]),
					.out_queue_in				(connect_out_queue_in[j][i]),
					.out_queue_filled_next		(connect_out_queue_filled_next[j][i]),
					.debug_q1_length				(queue_length[2*i]),
					.debug_q2_length				(queue_length[2*i+1])
					);
			end
			else if( j == K_BIT_2 -1 ) begin
				// special case: the last row
				sorter_node_interm #(.QUEUE_LENGTH_BITS(SORTER_INNER_QUEUE_LENGTH_BITS), .OVERFLOW_PADDING(BRAM_OVERFLOW_PADDING), .K_BITS(K_BITS) )  sorter_node_interm_last_row(
					.clk						(clk),
					.rst_n						(rst_n),
					.um_done					(um_done),
					.um_activated				(um_activated),
					.sorter_done				(sorter_done),

					.in1_queue_sorter_done		(connect_local_sorter_done[j-1][2*i]),
					.in1_queue_write_enable		(connect_out_queue_write_enable[j-1][2*i]),
					.in1_queue_in				(connect_out_queue_in[j-1][2*i]),
					.in1_queue_filled_next		(connect_out_queue_filled_next[j-1][2*i]),

					.in2_queue_sorter_done		(connect_local_sorter_done[j-1][2*i+1]),
					.in2_queue_write_enable		(connect_out_queue_write_enable[j-1][2*i+1]),
					.in2_queue_in				(connect_out_queue_in[j-1][2*i+1]),
					.in2_queue_filled_next		(connect_out_queue_filled_next[j-1][2*i+1]),

					.out_queue_write_enable		(out_queue_write_enable_intermediate),
					.out_queue_in				(out_queue_buf_in_intermediate),
					.out_queue_filled_next		(out_queue_buf_filled_next || out_edge_debug_buf_filled_next),
                    .local_sorter_done          (sorter_done)
				);
			end
			else begin
				sorter_node_interm #(.QUEUE_LENGTH_BITS(SORTER_INNER_QUEUE_LENGTH_BITS), .OVERFLOW_PADDING(BRAM_OVERFLOW_PADDING), .K_BITS(K_BITS) ) sorter_node_interm(
					.clk						(clk),
					.rst_n						(rst_n),
					.um_done					(um_done),
					.um_activated				(um_activated),
					.sorter_done				(sorter_done),

					.in1_queue_sorter_done		(connect_local_sorter_done[j-1][2*i]),	
					.in1_queue_write_enable		(connect_out_queue_write_enable[j-1][2*i]),
					.in1_queue_in				(connect_out_queue_in[j-1][ 2*i]),
					.in1_queue_filled_next		(connect_out_queue_filled_next[j-1][ 2*i]),

					.in2_queue_sorter_done		(connect_local_sorter_done[j-1][2*i+1]),
					.in2_queue_write_enable		(connect_out_queue_write_enable[j-1][ 2*i+1]),
					.in2_queue_in				(connect_out_queue_in[j-1][2*i+1]),
					.in2_queue_filled_next		(connect_out_queue_filled_next[j-1][2*i+1]),

					.out_queue_write_enable		(connect_out_queue_write_enable[j][i]),
					.out_queue_in				(connect_out_queue_in[j][i]),
					.out_queue_filled_next		(connect_out_queue_filled_next[j][i]),
                    .local_sorter_done          (connect_local_sorter_done[j][i])
				);
			end

		end
	end
end
endgenerate


/*
// high, if we do not output data, but we could
reg				do_not_work_valid;
reg [31:0]		do_not_work_cycle_counter;
// high, if we hava data, but cannot output data
reg				can_not_output_valid;
reg [31:0]		can_not_output_cycle_counter;
// high, if we do not have data, but could output
reg				can_not_input_valid;
reg [31:0]		can_not_input_cycle_counter;


always @(posedge clk) begin
	if (~rst_n | um_done | ~um_activated) begin
		do_not_work_valid <= 0;
		do_not_work_cycle_counter <= 0;
		can_not_output_valid <= 0;
		can_not_output_cycle_counter <= 0;
		can_not_input_valid <= 0;
		can_not_input_cycle_counter <= 0;
	end
	else begin
		do_not_work_valid <= 0;
		can_not_output_valid <= 0;
		can_not_input_valid <= 0;

		if( ~out_queue_buf_filled_next && ~out_queue_write_enable ) begin
			do_not_work_valid<= 1;
			do_not_work_cycle_counter <= do_not_work_cycle_counter + 1;
		end

		if( out_queue_buf_filled_next && debug_n_edges_in_normal > debug_n_edges_out_normal ) begin
			can_not_output_valid <= 1;
			can_not_output_cycle_counter <= can_not_output_cycle_counter  + 1;
		end

		if( ~out_queue_buf_filled_next && debug_n_edges_in_normal == debug_n_edges_out_normal ) begin
			can_not_input_valid <= 1;
			can_not_input_cycle_counter <= can_not_input_cycle_counter + 1;
		end
	end
end
*/



always @(posedge clk) begin
	debug_sorter <= { 256'b0, 
						queue_length[3], queue_length[2], queue_length[1], queue_length[0],
							debug_n_edges_in_normal, debug_n_edges_in_flush,
							debug_n_edges_out_normal, debug_n_edges_out_flush, 
						 	32'b0, 32'b0};
end

endmodule



`default_nettype wire









