
`default_nettype none




module sorter_node_interm #(parameter QUEUE_LENGTH_BITS = 4, parameter OVERFLOW_PADDING= 6, parameter K_BITS = 3 ) (
	//---------------------- GENERAL VARIABLES---------------------------//
	input   wire                                   	clk,							// clock
    input   wire                                   	rst_n,							// Stays low as long as AFU is idle
	input 	wire 									um_done,						// true, if we are done processing (global)
	input 	wire 									um_activated,					// true, if the afu is activated
	input 	wire									sorter_done,

	//---------------------- VARIABLES FOR THE INPUT DATA (predecessor) ---------------------------//
	input wire									in1_queue_sorter_done,
	input wire									in1_queue_write_enable,		// from our predecessor 1
	input wire[95:0]							in1_queue_in,				// from our predecessor 1
	output reg									in1_queue_filled_next,		// to our predecessor 1

	input wire									in2_queue_sorter_done,
	input wire									in2_queue_write_enable,		// from our predecessor 2
	input wire[95:0]							in2_queue_in,				// from our predecessor 2
	output reg									in2_queue_filled_next,		// to our predecessor 2

	//---------------------- VARIABLES FOR THE OUTPUT QUEUE (successor) ---------------------------//
	output reg									out_queue_write_enable,		// to our successor
	output reg[95:0]							out_queue_in,				// to our successor
	input wire									out_queue_filled_next,		// from our successor	

    output reg                                  local_sorter_done
);

`ifndef SORTER_EDGE_MIN_2

`define SORTER_EDGE_MIN_2(v1, v2) ( ( v1[95:64+K_BITS] < v2[95:64+K_BITS] ) ? 0 :  ( ( v1[95:64+K_BITS] == v2[95:64+K_BITS] ) & ( v1[63:32] < v2[63:32] ) ) ? 0 : ( ( v1[95:64+K_BITS] == v2[95:64+K_BITS] ) & ( v1[63:32] == v2[63:32] ) & ( v1[95:64] < v2[95:64] )  ) ? 0 : 1 )

`endif




wire q1_m_axis_valid;
reg q1_m_axis_ready;
wire [95:0] q1_m_axis_data;
wire [QUEUE_LENGTH_BITS-1:0] q1_data_count;
reg q1_data_buffer_1_valid;
reg q1_data_buffer_2_valid;
reg q1_data_buffer_3_valid;
reg [95:0] q1_data_buffer_1;
reg [95:0] q1_data_buffer_2;
reg [95:0] q1_data_buffer_3;

nlb_axififo #(.BUS_SIZE_ADDR(QUEUE_LENGTH_BITS), .BUS_SIZE_DATA(96), .PROG_FULL_THRESH(0 ) ) q1_axififo	(
    .clk (clk),
    .rst(~rst_n),
    .s_axis_valid(in1_queue_write_enable),
    .s_axis_ready(),
    .s_axis_data(in1_queue_in),
    .s_axis_last(1'b0),
    .m_axis_valid(q1_m_axis_valid),
    .m_axis_ready(q1_m_axis_ready),
    .m_axis_data(q1_m_axis_data),
    .m_axis_last(),
    .data_count(q1_data_count)
);

wire q2_m_axis_valid;
reg q2_m_axis_ready;
wire [95:0] q2_m_axis_data;
wire [QUEUE_LENGTH_BITS-1:0] q2_data_count;
reg q2_data_buffer_1_valid;
reg q2_data_buffer_2_valid;
reg q2_data_buffer_3_valid;

reg [95:0] q2_data_buffer_1;
reg [95:0] q2_data_buffer_2;
reg [95:0] q2_data_buffer_3;

nlb_axififo #(.BUS_SIZE_ADDR(QUEUE_LENGTH_BITS), .BUS_SIZE_DATA(96), .PROG_FULL_THRESH(0 ) ) q2_axififo	(
    .clk (clk),
    .rst(~rst_n),
    .s_axis_valid(in2_queue_write_enable),
    .s_axis_ready(),
    .s_axis_data(in2_queue_in),
    .s_axis_last(1'b0),
    .m_axis_valid(q2_m_axis_valid),
    .m_axis_ready(q2_m_axis_ready),
    .m_axis_data(q2_m_axis_data),
    .m_axis_last(),
    .data_count(q2_data_count)
);


reg[31:0] in1_debug_n_out_normal;
reg[31:0] in1_debug_n_out_flush;

reg[31:0] in2_debug_n_out_normal;
reg[31:0] in2_debug_n_out_flush;

// track debug infos, and the filled state of the queues
always @(posedge clk) begin
	if (~rst_n | um_done | ~um_activated) begin
		in1_queue_filled_next <= 0;
		in2_queue_filled_next <= 0;

		in1_debug_n_out_normal <= 0;
		in1_debug_n_out_flush <= 0;

		in2_debug_n_out_normal <= 0;
		in2_debug_n_out_flush <= 0;
	end
	else begin
		if( in1_queue_write_enable ) begin
			if( in1_queue_in[95:64] != in1_queue_in[63:32] ) begin
				in1_debug_n_out_normal <= in1_debug_n_out_normal + 1;
			end
			else begin
				in1_debug_n_out_flush <= in1_debug_n_out_flush + 1;
			end
		end
		in1_queue_filled_next <= q1_data_count  + OVERFLOW_PADDING > 2**QUEUE_LENGTH_BITS;
		
		if( in2_queue_write_enable ) begin
			if( in2_queue_in[95:64] != in2_queue_in[63:32] ) begin
				in2_debug_n_out_normal <= in2_debug_n_out_normal + 1;
			end
			else begin
				in2_debug_n_out_flush <= in2_debug_n_out_flush + 1;
			end
		end
		in2_queue_filled_next <= q2_data_count + OVERFLOW_PADDING >= 2**QUEUE_LENGTH_BITS;
	end

end

reg [31:0]	debug_num_edges;
reg [5:0] 	debug_case;

always @(posedge clk) begin
    if (~rst_n | um_done | ~um_activated) begin
		out_queue_write_enable <= 0;
		out_queue_in <= 0;

        local_sorter_done <= 0;
		debug_num_edges <= 0;
		
		q1_data_buffer_1_valid <= 0;
		q1_data_buffer_2_valid <= 0;
		q1_data_buffer_3_valid <= 0;
		q2_data_buffer_1_valid <= 0;
		q2_data_buffer_2_valid <= 0;
		q2_data_buffer_3_valid <= 0;

		// drain queues
		if( (um_done || ~um_activated) && rst_n ) begin
			q1_m_axis_ready <= 1;
			q2_m_axis_ready <= 1;
		end
		else begin
			q1_m_axis_ready <= 0;
			q2_m_axis_ready <= 0;
		end

		debug_case <= 0;
	end
	else begin
		out_queue_write_enable <= 0;

		if( ~sorter_done ) begin

            if( ( q1_data_count + q1_data_buffer_1_valid+ q1_data_buffer_2_valid + q1_data_buffer_3_valid) == 0 &&
				 ( q2_data_count + q2_data_buffer_1_valid + q2_data_buffer_2_valid + q2_data_buffer_3_valid) == 0 && ( in1_queue_sorter_done & in2_queue_sorter_done )  ) begin
                local_sorter_done <= 1;
				debug_case <= 1;
            end
			// if a) we cannot write out or b) a not-done input-queue has 0 input, then we must wait till we get more data: handle the updates
			else if( out_queue_filled_next || 
					(!in1_queue_sorter_done && ( q1_data_count + q1_data_buffer_1_valid+ q1_data_buffer_2_valid + q1_data_buffer_3_valid) == 0 ) ||
					 ( !in2_queue_sorter_done && ( q2_data_count + q2_data_buffer_1_valid + q2_data_buffer_2_valid + q2_data_buffer_3_valid) == 0) )begin
				debug_case <= 2;

				if( q1_m_axis_valid && q1_m_axis_ready ) begin
					if( ~q1_data_buffer_1_valid ) begin
						q1_data_buffer_1_valid <= 1;
						q1_data_buffer_1 <= q1_m_axis_data;
					end
					else if( ~q1_data_buffer_2_valid ) begin
						q1_data_buffer_2_valid <= 1;
						q1_data_buffer_2 <= q1_m_axis_data;
					end
					else if( ~q1_data_buffer_3_valid ) begin
						q1_data_buffer_3_valid <= 1;
						q1_data_buffer_3 <= q1_m_axis_data;
						q1_m_axis_ready <= 0;
					end
				end

				if( q2_m_axis_valid && q2_m_axis_ready ) begin
					if( ~q2_data_buffer_1_valid ) begin
						q2_data_buffer_1_valid <= 1;
						q2_data_buffer_1 <= q2_m_axis_data;
					end
					else if( ~q2_data_buffer_2_valid ) begin
						q2_data_buffer_2_valid <= 1;
						q2_data_buffer_2 <= q2_m_axis_data;
					end
					else if( ~q2_data_buffer_3_valid ) begin
						q2_data_buffer_3_valid <= 1;
						q2_data_buffer_3 <= q2_m_axis_data;
						q2_m_axis_ready <= 0;
					end
				end
			end
			// we have data in both: decide what we push next
			else if( ( q1_data_count + q1_data_buffer_1_valid) > 0 && ( q2_data_count + q2_data_buffer_1_valid) > 0  ) begin

				debug_case <= 3;
				if( q1_data_buffer_1_valid && q2_data_buffer_1_valid ) begin
					if(  ( `SORTER_EDGE_MIN_2(q1_data_buffer_1, q2_data_buffer_1)  ) == 0   ) begin
						// output from q1
						out_queue_in <= q1_data_buffer_1;
						out_queue_write_enable <= 1;
						debug_num_edges <= debug_num_edges  + 1;

						// handle update for q1
						if( q1_data_buffer_2_valid ) begin
							q1_data_buffer_1 <= q1_data_buffer_2;
							if( q1_data_buffer_3_valid ) begin
								q1_data_buffer_2 <= q1_data_buffer_3;
								if(  q1_m_axis_valid && q1_m_axis_ready ) begin
									q1_data_buffer_3 <= q1_m_axis_data;
									q1_m_axis_ready <= 0;
								end
								else begin
									q1_data_buffer_3_valid <= 0;
									q1_m_axis_ready <= 1;
								end
							end
							else begin
								q1_m_axis_ready <= 1;
								if(  q1_m_axis_valid && q1_m_axis_ready ) begin
									q1_data_buffer_2 <= q1_m_axis_data;
								end
								else begin
									q1_data_buffer_2_valid <= 0;
								end
							end
						end
						else begin
							q1_m_axis_ready <= 1;
							if(  q1_m_axis_valid && q1_m_axis_ready ) begin
								q1_data_buffer_1 <= q1_m_axis_data;
							end
							else begin
								q1_data_buffer_1_valid <= 0;
							end
						end


						// handle update for q2
						if( q2_m_axis_valid && q2_m_axis_ready ) begin
							if( ~q2_data_buffer_1_valid ) begin
								q2_data_buffer_1_valid <= 1;
								q2_data_buffer_1 <= q2_m_axis_data;
							end
							else if( ~q2_data_buffer_2_valid ) begin
								q2_data_buffer_2_valid <= 1;
								q2_data_buffer_2 <= q2_m_axis_data;
							end
							else if( ~q2_data_buffer_3_valid ) begin
								q2_data_buffer_3_valid <= 1;
								q2_data_buffer_3 <= q2_m_axis_data;
								q2_m_axis_ready <= 0;
							end
						end
						
						
					end
					else begin
							// output from q2
							out_queue_in <= q2_data_buffer_1;
							out_queue_write_enable <= 1;
							debug_num_edges <= debug_num_edges + 1;

							// handle update for q1
							if( q1_m_axis_valid && q1_m_axis_ready ) begin
								if( ~q1_data_buffer_1_valid ) begin
									q1_data_buffer_1_valid <= 1;
									q1_data_buffer_1 <= q1_m_axis_data;
								end
								else if( ~q1_data_buffer_2_valid ) begin
									q1_data_buffer_2_valid <= 1;
									q1_data_buffer_2 <= q1_m_axis_data;
								end
								else if( ~q1_data_buffer_3_valid ) begin
									q1_data_buffer_3_valid <= 1;
									q1_data_buffer_3 <= q1_m_axis_data;
									q1_m_axis_ready <= 0;
								end
							end


							// handle update for q2
							if( q2_data_buffer_2_valid ) begin
								q2_data_buffer_1 <= q2_data_buffer_2;
								if( q2_data_buffer_3_valid ) begin
									q2_data_buffer_2 <= q2_data_buffer_3;
									if(  q2_m_axis_valid && q2_m_axis_ready ) begin
										q2_data_buffer_3 <= q2_m_axis_data;
										q2_m_axis_ready <= 0;
									end
									else begin
										q2_data_buffer_3_valid <= 0;
										q2_m_axis_ready <= 1;
									end
								end
								else begin
									q2_m_axis_ready <= 1;
									if(  q2_m_axis_valid && q2_m_axis_ready ) begin
										q2_data_buffer_2 <= q2_m_axis_data;
									end
									else begin
										q2_data_buffer_2_valid <= 0;
									end
								end
							end
							else begin
								q2_m_axis_ready <= 1;
								if(  q2_m_axis_valid && q2_m_axis_ready ) begin
									q2_data_buffer_1 <= q2_m_axis_data;
								end
								else begin
									q2_data_buffer_1_valid <= 0;
								end
							end
					end

				end
				else begin
					// nothing is written out, since we have not data for both
					// q1_m_axis_ready <= 1;
					// q2_m_axis_ready <= 1;
					

					if( q1_m_axis_valid && q1_m_axis_ready ) begin
						if( ~q1_data_buffer_1_valid ) begin
							q1_data_buffer_1_valid <= 1;
							q1_data_buffer_1 <= q1_m_axis_data;
						end
						else if( ~q1_data_buffer_2_valid ) begin
							q1_data_buffer_2_valid <= 1;
							q1_data_buffer_2 <= q1_m_axis_data;
						end
						else if( ~q1_data_buffer_3_valid ) begin
							q1_data_buffer_3_valid <= 1;
							q1_data_buffer_3 <= q1_m_axis_data;
							q1_m_axis_ready <= 0;
						end
					end
					else if( ~q1_data_buffer_3_valid || ~q1_data_buffer_2_valid || ~q1_data_buffer_1_valid ) begin
						q1_m_axis_ready <= 1;
					end


					if( q2_m_axis_valid && q2_m_axis_ready ) begin
						if( ~q2_data_buffer_1_valid ) begin
							q2_data_buffer_1_valid <= 1;
							q2_data_buffer_1 <= q2_m_axis_data;
						end
						else if( ~q2_data_buffer_2_valid ) begin
							q2_data_buffer_2_valid <= 1;
							q2_data_buffer_2 <= q2_m_axis_data;
						end
						else if( ~q2_data_buffer_3_valid ) begin
							q2_data_buffer_3_valid <= 1;
							q2_data_buffer_3 <= q2_m_axis_data;
							q2_m_axis_ready <= 0;
						end
					end
					else if( ~q2_data_buffer_3_valid || ~q2_data_buffer_2_valid || ~q2_data_buffer_1_valid ) begin
						q2_m_axis_ready <= 1;
					end


				end
				
			end
			
			// if 0 is done, we can simply take from the other one -> note the order of the cases!!
			else if( ( q1_data_count + q1_data_buffer_1_valid) > 0 ) begin // then, we can read from queue 1

						debug_case <= 6;

				if( q1_data_buffer_1_valid ) begin

						// output from q1
						out_queue_in <= q1_data_buffer_1;
						out_queue_write_enable <= 1;
						debug_num_edges <= debug_num_edges  + 1;

						// handle update for q1
						if( q1_data_buffer_2_valid ) begin
							q1_data_buffer_1 <= q1_data_buffer_2;
							if( q1_data_buffer_3_valid ) begin
								q1_data_buffer_2 <= q1_data_buffer_3;
								if(  q1_m_axis_valid && q1_m_axis_ready ) begin
									q1_data_buffer_3 <= q1_m_axis_data;
									q1_m_axis_ready <= 0;
								end
								else begin
									q1_data_buffer_3_valid <= 0;
									q1_m_axis_ready <= 1;
								end
							end
							else begin
								q1_m_axis_ready <= 1;
								if(  q1_m_axis_valid && q1_m_axis_ready ) begin
									q1_data_buffer_2 <= q1_m_axis_data;
								end
								else begin
									q1_data_buffer_2_valid <= 0;
								end
							end
						end
						else begin
							q1_m_axis_ready <= 1;
							if(  q1_m_axis_valid && q1_m_axis_ready ) begin
								q1_data_buffer_1 <= q1_m_axis_data;
							end
							else begin
								q1_data_buffer_1_valid <= 0;
							end
						end
				end
				else begin
						if( q1_m_axis_valid && q1_m_axis_ready ) begin
							if( ~q1_data_buffer_1_valid ) begin
								q1_data_buffer_1_valid <= 1;
								q1_data_buffer_1 <= q1_m_axis_data;
							end
							else if( ~q1_data_buffer_2_valid ) begin
								q1_data_buffer_2_valid <= 1;
								q1_data_buffer_2 <= q1_m_axis_data;
							end
							else if( ~q1_data_buffer_3_valid ) begin
								q1_data_buffer_3_valid <= 1;
								q1_data_buffer_3 <= q1_m_axis_data;
								q1_m_axis_ready <= 0;
							end
						end
						else if( ~q1_data_buffer_3_valid || ~q1_data_buffer_2_valid || ~q1_data_buffer_1_valid ) begin
							q1_m_axis_ready <= 1;
						end
				end

 
			end
			else if( ( q2_data_count + q2_data_buffer_1_valid) > 0 ) begin // then, we read from queue 2

							debug_case <= 7;

						if( q2_data_buffer_1_valid ) begin

							// output from q2
							out_queue_in <= q2_data_buffer_1;
							out_queue_write_enable <= 1;
							debug_num_edges <= debug_num_edges + 1;

							// handle update for q2
							if( q2_data_buffer_2_valid ) begin
								q2_data_buffer_1 <= q2_data_buffer_2;
								if( q2_data_buffer_3_valid ) begin
									q2_data_buffer_2 <= q2_data_buffer_3;
									if(  q2_m_axis_valid && q2_m_axis_ready ) begin
										q2_data_buffer_3 <= q2_m_axis_data;
										q2_m_axis_ready <= 0;
									end
									else begin
										q2_data_buffer_3_valid <= 0;
										q2_m_axis_ready <= 1;
									end
								end
								else begin
									q2_m_axis_ready <= 1;
									if(  q2_m_axis_valid && q2_m_axis_ready ) begin
										q2_data_buffer_2 <= q2_m_axis_data;
									end
									else begin
										q2_data_buffer_2_valid <= 0;
									end
								end
							end
							else begin
								q2_m_axis_ready <= 1;
								if(  q2_m_axis_valid && q2_m_axis_ready ) begin
									q2_data_buffer_1 <= q2_m_axis_data;
								end
								else begin
									q2_data_buffer_1_valid <= 0;
								end
							end

						end
						else begin
							if( q2_m_axis_valid && q2_m_axis_ready ) begin
								if( ~q2_data_buffer_1_valid ) begin
									q2_data_buffer_1_valid <= 1;
									q2_data_buffer_1 <= q2_m_axis_data;
								end
								else if( ~q2_data_buffer_2_valid ) begin
									q2_data_buffer_2_valid <= 1;
									q2_data_buffer_2 <= q2_m_axis_data;
								end
								else if( ~q2_data_buffer_3_valid ) begin
									q2_data_buffer_3_valid <= 1;
									q2_data_buffer_3 <= q2_m_axis_data;
									q2_m_axis_ready <= 0;
								end
							end
							else if( ~q2_data_buffer_3_valid || ~q2_data_buffer_2_valid || ~q2_data_buffer_1_valid ) begin
								q2_m_axis_ready <= 1;
							end
						end


			end

		end
	end
end

endmodule






`default_nettype wire





