
#ifdef FPGA_FLAG


#include "mwm_crouch_csr_ram.hpp"

MWMCrouchCSRRam::MWMCrouchCSRRam()
{
    std::cout << "create new FPGA object" << std::endl;
    this->myfpga = new FPGA();
    std::cout << "new FPGA object created" << std::endl;
}

MWMCrouchCSRRam::~MWMCrouchCSRRam()
{
    if(this->myfpga != nullptr)
    {
        delete(this->myfpga);
        this->myfpga = nullptr;
    }
}

void* MWMCrouchCSRRam::solve(void* pinput, Timing* timer, RunParams* rp)
{
    uint32_t* ptrs ;
    uint64_t in_ptrs_num_cl;
    uint32_t* edges;
    uint64_t in_edges_num_cl;

    WEdgeList* input  = nullptr;
    if( pinput != nullptr)
    {
        input = (WEdgeList*) pinput;
        Util::convert_to_csr_fpga_format(input, &ptrs, &in_ptrs_num_cl, &edges, &in_edges_num_cl );
    }
    else
    {
        input = (WEdgeList*) malloc(sizeof(WEdgeList));
        input->wedge = new std::vector<WEdge>();

        WEdge e1; e1.u = 1; e1.v = 2; e1.weight = 1;
        WEdge e2; e2.u = 2; e2.v = 3; e2.weight = 1;
        input->wedge->push_back(e1);
        input->wedge->push_back(e2);
        input->m = 2;
        input->n = 3;

        in_ptrs_num_cl = 1;
        in_edges_num_cl = 1;

        ptrs = (uint32_t*) malloc(sizeof(uint32_t) * 100);
        ptrs[0] = 0;
        ptrs[1] = 0;
        ptrs[2] = 2;

        edges = (uint32_t*) malloc(sizeof(uint32_t) * 100);
        edges[0] = 2;
        edges[1] = 1;
        edges[2] = 3;
        edges[3] = 1;
    }

    // 1. convert to a format that is suitable for the FPGA: in the "cache-line" format

    const uint64_t num_levels = 32;
    const uint64_t in_edges_num_edges = input->m;
    const uint64_t in_edges_size_buffer_in_ints = in_edges_num_cl*512/32;
    const uint64_t in_edges_size_buffer_in_bytes = in_edges_size_buffer_in_ints*sizeof(uint32_t);

    const uint64_t in_ptrs_num_ptrs = input->n;
    const uint64_t in_ptrs_size_buffer_in_ints = in_ptrs_num_cl*512/32;
    const uint64_t in_ptrs_size_buffer_in_bytes = in_ptrs_size_buffer_in_ints*sizeof(uint32_t);

    const uint64_t out_edges_num_edges =  std::min(in_edges_num_edges, ( (uint64_t) input->n/2 + 2) * num_levels);
    const uint64_t out_edges_num_cl = out_edges_num_edges*128/512 + 1;
    const uint64_t out_edges_size_buffer_in_ints = out_edges_num_cl*512/32;
    const uint64_t out_edges_size_buffer_in_bytes = out_edges_num_cl*512/8;

    const uint64_t mcm_bits_num_bits = input->n * num_levels;
    const uint64_t mcm_bits_num_cl = mcm_bits_num_bits / 512 + 1;
    const uint64_t mcm_bits_size_buffer_in_ints = mcm_bits_num_cl * 512/32;
    const uint64_t mcm_bits_size_buffer_in_bytes = mcm_bits_num_cl * 512/8;

    const uint64_t sorted_edges_num_cl = in_edges_num_cl*2;
    const uint64_t sorted_edges_num_edges = in_edges_num_edges;
    const uint64_t sorted_edges_size_buffer_in_ints = sorted_edges_num_cl*512/32;
    const uint64_t sorted_edges_size_buffer_in_bytes = sorted_edges_size_buffer_in_ints*sizeof(uint32_t);;

#ifndef PERFORMANCE
    std::cout << "in_edges_num_cl="<<in_edges_num_cl<< std::endl;
    std::cout << "in_edges_num_edges="<<in_edges_num_edges << std::endl;
    std::cout << "in_edges_size_buffer_in_ints="<<in_edges_size_buffer_in_ints<< std::endl;
    std::cout << "in_edges_size_buffer_in_bytes="<<in_edges_size_buffer_in_bytes<< std::endl;

    std::cout << "in_ptrs_num_cl="<<in_ptrs_num_cl << std::endl;
    std::cout << "in_ptrs_num_ptrs="<<in_ptrs_num_ptrs<< std::endl;
    std::cout << "in_ptrs_size_buffer_in_ints="<<in_ptrs_size_buffer_in_ints<<std::endl;
    std::cout << "in_ptrs_size_buffer_in_bytes="<< in_ptrs_size_buffer_in_bytes << std::endl;

    std::cout << "out_edges_num_cl="<< out_edges_num_cl << std::endl;
    std::cout << "out_edges_num_edges="<< out_edges_num_edges<< std::endl;
    std::cout << "out_edges_size_buffer_in_ints="<<out_edges_size_buffer_in_ints<< std::endl;
    std::cout << "out_edges_size_buffer_in_bytes="<<out_edges_size_buffer_in_bytes<< std::endl;

    std::cout << "mcm_bits_num_bits="<< mcm_bits_num_bits << std::endl;
    std::cout << "mcm_bits_num_cl="<< mcm_bits_num_cl << std::endl;
    std::cout << "mcm_bits_size_buffer_in_ints="<< mcm_bits_size_buffer_in_ints << std::endl;
    std::cout << "mcm_bits_size_buffer_in_bytes="<< mcm_bits_size_buffer_in_bytes<< std::endl;


    std::cout << "sorted_edges_num_cl="<< sorted_edges_num_cl << std::endl;
    std::cout << "sorted_edges_num_edges="<< sorted_edges_num_edges << std::endl;
    std::cout << "sorted_edges_size_buffer_in_ints=" << sorted_edges_size_buffer_in_ints << std::endl;
    std::cout << "sorted_edges_size_buffer_in_bytes="<<sorted_edges_size_buffer_in_bytes << std::endl;
#endif

    // allocate the buffer on the fpga
    uint32_t* data_edges_in   = reinterpret_cast<uint32_t*>( myfpga->malloc(in_edges_size_buffer_in_bytes ));
    uint32_t* data_ptrs_in = reinterpret_cast<uint32_t*>( myfpga->malloc(in_ptrs_size_buffer_in_bytes ));
    uint32_t* data_edges_out  = reinterpret_cast<uint32_t*>( myfpga->malloc(out_edges_size_buffer_in_bytes));
    uint32_t* data_mcm_bits = reinterpret_cast<uint32_t*>(myfpga->malloc( mcm_bits_size_buffer_in_bytes));
    uint32_t* data_sorted_edges_out = reinterpret_cast<uint32_t*>( myfpga->malloc(sorted_edges_size_buffer_in_bytes ));
    volatile uint32_t* data_debug = reinterpret_cast<volatile uint32_t*>( myfpga->malloc(512/8*64 ));

    // copy the input edges
    for(uint64_t i = 0; i < in_edges_size_buffer_in_ints; i++)
    {
        data_edges_in[i] = edges[i];
    }

    // copy the input pointers
    for(uint64_t i = 0; i < in_ptrs_size_buffer_in_ints; i++)
    {
        data_ptrs_in[i] = ptrs[i];
    }


    for(uint64_t i = 0; i < out_edges_size_buffer_in_ints; i++)
    {
        data_edges_out[i] = 0;
    }

    for(uint64_t i = 0; i < mcm_bits_size_buffer_in_ints; i ++)
    {
        data_mcm_bits[i] = 0;
    }


    for(uint64_t i = 0; i < sorted_edges_size_buffer_in_ints; i++)
    {
       data_sorted_edges_out[i] = 0;
    }

    free(edges);
    free(ptrs);

    // 2. create and call the function
    MWM_CSR_RAM_AFU_CONFIG * afu_cfg = (struct MWM_CSR_RAM_AFU_CONFIG*)(myfpga->malloc( sizeof(MWM_CSR_RAM_AFU_CONFIG) ));

    afu_cfg->pEdges = data_edges_in;
    afu_cfg->pPtrs = data_ptrs_in;
    afu_cfg->pDestEdges  = data_edges_out;
    afu_cfg->pDestMCMBits = data_mcm_bits;
    afu_cfg->num_ptrs = in_ptrs_num_ptrs;
    afu_cfg->num_edges = in_edges_num_edges;
    afu_cfg->pDestDebugEdges = data_sorted_edges_out;
    afu_cfg->pDebug = (void*)data_debug;

#ifndef PERFORMANCE
    printf("afu_cfg size in bytes: %lu\n", sizeof(MWM_CSR_RAM_AFU_CONFIG) );
    printf("afu cfg struct: %p, data_ptrs_in: %p, data_edges_in: %p, data_edges_out: %p, data_mcm_bits: %p, data_sorted_edges_out %p, data_debug %p \n", afu_cfg, data_ptrs_in, data_edges_in, data_edges_out, data_mcm_bits, data_sorted_edges_out, data_debug); fflush(stdout);

    printf("afu_cfg struct:  %p\n", afu_cfg);
    printf("afu_cfg->pPtrs %p\n",  afu_cfg->pPtrs);
    printf("afu_cfg->pEdges %p\n", afu_cfg->pEdges);
    printf("afu_cfg->pDestEdges %p\n",  afu_cfg->pDestEdges);
    printf("afu_cfg->pDestMCMBits %p\n", afu_cfg->pDestMCMBits);
    printf("afu_cfg->pDestDebugEdges %p\n",  afu_cfg->pDestDebugEdges);
    printf("afu_cfg->pDebug %p\n",  afu_cfg->pDebug);

    puts("Create Op"); fflush(stdout);
#endif
    auto start_time = std::chrono::high_resolution_clock::now();

    // destruction of record is done by Fthread
    auto mwmThreadRecOp =  new FthreadRec(myfpga, MWM_CROUCH_CSR_RAM_OP, reinterpret_cast<unsigned char*>(afu_cfg), sizeof(MWM_CSR_RAM_AFU_CONFIG) );
#ifndef PERFORMANCE
    std::cout << "Start of operation" << std::endl;
#endif
    Fthread mwmThreadOp(mwmThreadRecOp);
#ifndef PERFORMANCE
    std::cout <<"Wait for termination "<< std::endl;
#endif

#ifndef PERFORMANCE
    uint32_t i = 0;
    int sleeps = 0;

    unsigned char state = 0;
    do
    {
        puts("----------");
        print_bram_mcm_bit_receiver(data_debug +  0);
        print_bram_mcm_bit_requester(data_debug + 16);
        print_edge_processor(data_debug + 32);
        print_edge_receiver(data_debug + 48);
        print_edge_requester(data_debug + 64);
        print_edge_writer(data_debug + 80);
        print_mcm_bit_ack_receiver(data_debug + 96);
        print_mcm_bit_receiver(data_debug + 112);
        print_mcm_bit_requester(data_debug + 128);
        print_mcm_bit_writer(data_debug + 144);
        print_mwm_crouch_csr_ram(data_debug + 160);
        print_pointer_receiver(data_debug + 176);
        print_pointer_requester(data_debug + 192);
        print_request_interface(data_debug + 208);
        print_sorter(data_debug + 224);
        print_state_controller(data_debug + 240);
        print_write_interface(data_debug + 256);
        puts("----------");

        state = mwmThreadRecOp->get_status()->state;

        SleepNano( 500*1000*1000 ); // sleep for 0.5 sec

        sleeps++;
        if (sleeps >= 1200*2) { // 600sec = 10 min
            std::cout << "Killing AFU" << std::endl;
            myfpga->f_platform->srvHndle->m_pALIMMIOService->mmioWrite64(8 * (32 + 6), AAL::btUnsigned64bitInt(0));
            SleepMilli(1000);
            break;
        }

    }while(state != OPERATOR_DONE_STATE);

#endif
    mwmThreadOp.join();

#ifndef PERFORMANCE
    mwmThreadOp.printStatusLine();


    //-----------------------------------------------------------------------------------------------------------------------------------//
    auto   end_time    = std::chrono::high_resolution_clock::now();
    double execTime    = (std::chrono::duration_cast<std::chrono::microseconds>(end_time - start_time).count())/1000.0;
    double opLocalTime = mwmThreadOp.timing();
    double throughput  = 1000.0*in_edges_num_cl/execTime;


    printf("Process done: exec time: %.10f, %.10f, %.10f\n", execTime, opLocalTime, throughput);  fflush(stdout);
    puts("data_edges_out: ");
    for(uint64_t i = 0; i < out_edges_size_buffer_in_ints; i++)
    {
        printf("%d ", data_edges_out[i]);
    }
    puts("\n");


    puts("data_mcm_bits");
    for(uint64_t i = 0; i < mcm_bits_size_buffer_in_ints; i++ )
    {
        printf("%X ", data_mcm_bits[i]);
    }
    puts("\n");

    std::ofstream newFile1("fpga.edges_sorted.txt");
    puts("sorted edges out (written to fpga.edges_sorted.txt):");
    uint64_t edge_cnt = 0;
    for(uint64_t i = 0; i < sorted_edges_size_buffer_in_ints; i= i + 4)
    {
        if( data_sorted_edges_out[i+2] != data_sorted_edges_out[i+1] )
        {
            newFile1 << "Edge["<< edge_cnt <<"]: epoch="<< data_sorted_edges_out[i+3] <<", u="<< data_sorted_edges_out[i+2] <<", v="<< data_sorted_edges_out[i+1]<< ", w="<< data_sorted_edges_out[i+0] << std::endl;
            edge_cnt = edge_cnt  + 1;
        }
    }
    newFile1.close();

#endif

    WEdgeList* result = (WEdgeList*) malloc(sizeof(WEdgeList));
    result->wedge = new std::vector<WEdge>();

    auto bv = new BitVector(input->n + 2);

    // greedily create the matching, starting with the highest level

#ifndef PERFORMANCE
    std::ofstream newFile("fpga.out.txt");
    std::cout << "add: (write to fpga.out.txt) "<<std::endl;
#endif
    for(int64_t l = 512; l >= 0; l--)
    {
        for(uint64_t i = 0; i <  out_edges_size_buffer_in_ints/4; i++)
        {
            uint32_t eu = data_edges_out[i*4];
            uint32_t ev = data_edges_out[i*4+1];
            uint32_t ew = data_edges_out[i*4+2];
            uint32_t emax_pos = data_edges_out[i*4+3];

            if(emax_pos == l && eu != 0 && ev != 0 && bv->get(eu) == 0 && bv->get(ev) == 0)
            {
                WEdge we; we.u = eu; we.v = ev; we.weight = ew;
                bv->set_true(eu);
                bv->set_true(ev);
                result->wedge->push_back(we);
#ifndef PERFORMANCE
                newFile << "e.u="<<eu<<", e.v="<<ev<<", e.weight="<<ew<<", e.max_pos="<<emax_pos<<std::endl;
#endif
            }
        }
    }

#ifndef PERFORMANCE
    newFile.close();
    std::cout<< "end added"<<std::endl;
#endif

    result->m = result->wedge->size();
    result->n = input->n;

    delete(bv);

    myfpga->free(data_edges_in);
    myfpga->free(data_ptrs_in);
    myfpga->free(data_edges_out);
    myfpga->free(data_mcm_bits );
    myfpga->free(data_sorted_edges_out);
    myfpga->free( (void*)data_debug);

    return result;
}




double MWMCrouchCSRRam::get_guarantee(RunParams* rp)
{
    return 4.0f + rp->epsilon;
}

std::string MWMCrouchCSRRam::get_solver_name()
{
    return "mwmw_crouch_csr_ram";
}

double MWMCrouchCSRRam::get_min_epsilon()
{
    return 0.8f;
}

double MWMCrouchCSRRam::get_min_delta()
{
    return 0.0f;
}


void MWMCrouchCSRRam::print_bram_mcm_bit_receiver(volatile uint32_t* cl_start)
{
    std::cout << "bram_mcm_bit_receiver: edges_count="<< *cl_start<< ", ntotal_write="<< *(cl_start+1) <<", ntotal_read="<< *(cl_start+2) << std::endl;
}

void MWMCrouchCSRRam::print_bram_mcm_bit_requester(volatile uint32_t* cl_start)
{
    std::cout << "bram_mcm_bit_requester: num_cl_requested="<<*cl_start <<", num_usefull_bits_requested="<<*(cl_start +1)
              << ", num_usefull_bits_to_request="<< *(cl_start +2) <<", num_current_bits="<< *(cl_start +3) <<  std::endl;
}

void MWMCrouchCSRRam::print_edge_processor(volatile uint32_t* cl_start)
{
    std::cout << "edge_processor: stage_6_edges="<< *cl_start << ", stage_5_edges="<< *(cl_start+1)
              << ", stage_4_edges="<< *(cl_start + 2) << ", stage_3_edges="<< *(cl_start +3)
              << ", stage_1_edges="<< *(cl_start +5)
              << ", edgein_ntotal_write="<< *(cl_start+6) <<", edgein_ntotal_read="<< *(cl_start + 7)
              << ", mcmin_ntotal_write="<< *(cl_start +8) << ", mcmin_ntotal_read="<< *(cl_start+9)
              << ", offset_ntotal_write="<< *(cl_start +10) <<", offset_ntotal_read="<< *(cl_start +11)
              << ", mcmbram_ntotal_write="<< *(cl_start +12)<< ", mcmbram_ntotal_read="<< *(cl_start +13)
              << ", stage_5_one_hot_take_edge[31:0]="<< *(cl_start +14) << ", stage_5_one_hot_take_edge[63:32]="<< *(cl_start +15) << std::endl;
}

void MWMCrouchCSRRam::print_edge_receiver(volatile uint32_t * cl_start)
{
    std::cout << "edge_receiver: info_ntotal_write="<< *cl_start <<", info_ntotal_read="<< *(cl_start + 1)
              << ", cl_ntotal_write= "<< *(cl_start + 2) <<", cl_ntotal_read="<< *(cl_start + 3)
              << ", n_processed_edges="<< *(cl_start +4) <<", n_processed_edges_flush="<< *(cl_start +5)
              << ", debug_info_buf_data_count="<< *(cl_start +6) << ", debug_cl_buf_data_count="<< *(cl_start +7)
              << ", debug_case_error_flag_32b="<< *(cl_start +8) << ", debug_case="<<  *(cl_start +9)
              << ", debug_assign_error_flag_32b="<< *(cl_start +10) << ", debug_queue_overflow="<< *(cl_start +11)
              <<", debug_info_buf_valid="<< *(cl_start +12) <<", debug_cl_buf_valid="<< *(cl_start +13) << std::endl;
}

void MWMCrouchCSRRam::print_edge_requester(volatile uint32_t* cl_start)
{
    volatile uint16_t* cl_start_16 = (volatile uint16_t*)cl_start;

   std::cout << "edge_requester: stage_4_debug_num_flush_pointers="<< *cl_start<<", stage_1_k_to_request_valid_32b="<< *(cl_start +1)
              <<", stage_1_k_to_request_32b="<< *(cl_start +2) <<", stage_4_num_removed_pointers="<< *(cl_start +3)
             << ", stage_4_num_edge_requests="<<*(cl_start +4)<< ", stage_4_num_total_edges_requested="<< *(cl_start +5)
               << ", stage_4_debug_bram_1_valid=" << *(cl_start_16 + 12) <<", stage_4_debug_bram_2_valid="<< *(cl_start_16 +13) << ", stage_4_debug_case="<< *(cl_start_16 + 14) <<", stage_4_debug_rd_k="<< *(cl_start_16 + 15)
               <<", queue_read="<< *(cl_start +8) << ", queue_write="<< * (cl_start +9)
              << ", stage_1_state="<< *(cl_start_16 + 20) << ", stage_2_valid_debug="<< *(cl_start_16 + 21)  << ", stage_3_debug_case="<< *(cl_start_16 +22)
              << ", edge_length_debug_0="<< *(cl_start_16 + 24) <<", edge_length_debug_1="<< *(cl_start_16 + 25) <<", edge_length_debug_2="<< *(cl_start_16 + 26)<<", edge_length_debug_3=" << *(cl_start_16 + 27)
                <<", ptr_length_debug_0=" << *(cl_start_16 + 28)<< ", ptr_length_debug_1="<< *(cl_start_16 + 29)<<", ptr_length_debug_2=" << *(cl_start_16 + 30) << ", ptr_length_debug_3=" << *(cl_start_16 + 31) << std::endl;
}

void MWMCrouchCSRRam::print_edge_writer(volatile uint32_t* cl_start)
{
    std::cout << "edge_writer: cl_cnt="<< *cl_start<< ", edge_ntotal_write="<< *(cl_start+1)<<", edge_ntotal_read="<<*(cl_start + 2) << std::endl;
}

void MWMCrouchCSRRam::print_mcm_bit_ack_receiver(volatile uint32_t* cl_start)
{
    std::cout << "mcm_bit_ack_receiver: num_acks_received="<< *cl_start << std::endl;
}

void MWMCrouchCSRRam::print_mcm_bit_receiver(volatile uint32_t* cl_start)
{
    std::cout << "mcm_bit_receiver: num_resp_received="<< *cl_start << ", ntotal_write=" << *(cl_start +1) << ", ntotal_read="<< *(cl_start +2) << std::endl;
}

void MWMCrouchCSRRam::print_mcm_bit_requester(volatile uint32_t* cl_start)
{
    std::cout << "mcm_bit_requester: num_processed_edges="<< *cl_start << ", ntotal_write="<< *(cl_start +1)
              <<", ntotal_read="<< *(cl_start +2) << ", mcm_bit_wait_counter=" << *(cl_start +3) <<  std::endl;
}

void MWMCrouchCSRRam::print_mcm_bit_writer(volatile uint32_t* cl_start)
{
    std::cout << "mcm_bit_writer: mcm_bit_writer_num_writes="<< *cl_start << ", mcm_ntotal_write="<< *(cl_start +1) << ", mcm_ntotal_read="<< *(cl_start +2) << std::endl;
}

void MWMCrouchCSRRam::print_mwm_crouch_csr_ram(volatile uint32_t* cl_start)
{
    volatile uint64_t* cl_64_start = (uint64_t*) cl_start;
    std::cout << "mwm_crouch_csr_ram: ptr_base_addr="<< *cl_64_start<< ", edges_base_addr="<< *(cl_64_start+1)
              << ", out_edges_base_addr="<< *(cl_64_start+2) <<", out_mcm_bits_base_addr="<< *(cl_64_start+3)
              << ", out_edge_debug_base_addr="<< *(cl_64_start+4)<<", debug_base_addr="<< *(cl_64_start+5)
              << ", num_ptrs="<< *(cl_start +12) << ", num_edges="<< *(cl_start +13) << std::endl;
}

void MWMCrouchCSRRam::print_pointer_receiver(volatile uint32_t* cl_start)
{
    std::cout << "poniter_receiver: debug_num_flush_pointers="<< *cl_start << ", ptr_processed_cnt="<< *(cl_start +1) << ", in_queue_ntotal_write="<< *(cl_start +2) << ", in_queue_ntotal_read="<< *(cl_start +3) << ", debug_edges_count="<< *(cl_start +4)<< std::endl;
}

void MWMCrouchCSRRam::print_pointer_requester(volatile uint32_t* cl_start)
{
    std::cout << "pointer_requester: prq_num_req="<< *cl_start << std::endl;
}

void MWMCrouchCSRRam::print_request_interface(volatile uint32_t* cl_start)
{
    std::cout << "request_interface: mcm_bram_ntotal_write=" << *cl_start << ", mcm_bram_ntotal_read="<<*(cl_start +1)
              << ", mcm_ntotal_write="<< *(cl_start+2) << ", mcm_ntotal_read="<<*(cl_start +3)
              << ", ptr_ntotal_write="<< *(cl_start +4) << ", ptr_ntotal_read="<< *(cl_start +5)
              << ", edge_ntotal_write="<< *(cl_start +6) <<", edge_ntotal_read="<< *(cl_start +7)
              << ", debug_address_error32=" << *(cl_start + 8) <<", debug_case_error="<< *(cl_start + 9)
              << ", num_received_responses="<< *(cl_start + 10) <<", num_issued_requests="<< *(cl_start + 11) << std::endl;
}

void MWMCrouchCSRRam::print_sorter(volatile uint32_t* cl_start)
{
    volatile uint16_t* cl_start_16 = (volatile uint16_t*)cl_start;

    std::cout << "sorter: can_not_sort_32b="<< *cl_start
              << ", debug_n_edges_out_flush="<< *(cl_start +2) << ", debug_n_edges_out_normal="<< *(cl_start +3)
              <<", debug_n_edges_in_flush="<< *(cl_start +4) << ", debug_n_edges_in_normal="<<*(cl_start +5)
               << ", queue_length_0="<< *(cl_start_16 + 12) << ", queue_length_1="<< *(cl_start_16 + 13) << ", queue_length_2="<< *(cl_start_16 + 14) <<", queue_length_3="<< *(cl_start_16 + 15) << std::endl;
}

void MWMCrouchCSRRam::print_state_controller(volatile uint32_t* p_cl_start)
{
    volatile uint16_t* cl_start = (volatile uint16_t*)p_cl_start;
    volatile uint64_t* clk_cnt = ( (volatile uint64_t*) p_cl_start )  + 6;

    std::cout << "state_controller: mcm_bit_ack_receiver_epoch_done="<< *(cl_start+1) << ", bram_mcm_bit_receiver_epoch_done="<< *(cl_start +2)
              << ", bram_mcm_bit_requester_epoch_done="<< *(cl_start +3)
              << ", mcm_writer_epoch_done="<< *(cl_start +4) <<", edge_processor_epoch_done="<< *(cl_start +5)
              << ", mcm_bit_receiver_epoch_done="<< *(cl_start +6) <<", mcm_bit_requester_epoch_done="<< *(cl_start +7)
              << ", edge_writer_done="<< *(cl_start +10)<<", mcm_bit_ack_receiver_done="<< *(cl_start +11)
              << ", write_interface_done="<< *(cl_start +12)<< ", edge_processor_done="<< *(cl_start +13)
              << ", bram_mcm_bit_receiver_done="<< *(cl_start +14) <<", bram_mcm_bit_requester_done="<< *(cl_start +15)
              << ", mcm_bit_receiver_done="<< *(cl_start + 16) << ", mcm_bit_requester_done="<< *(cl_start +17)
              << ", sorter_done="<< *(cl_start + 19)
              << ", edge_receiver_done="<< *(cl_start +20) <<", edge_requester_done="<< *(cl_start +21)
              <<", pointer_receiver_done="<< *(cl_start +22) << ", pointer_requester_done="<< *(cl_start +23)
             << ", clk_counter="<< (*clk_cnt) << ", global_epoch="<< *(p_cl_start+14)
             << ", mcm_ack_cycle_count="<< *(p_cl_start+15) << std::endl;
}

void MWMCrouchCSRRam::print_write_interface(volatile uint32_t* cl_start)
{
    std::cout << "write_interface: edge_debug_ntotal_write="<< *cl_start << ", edge_debug_ntotal_read="<<*(cl_start +1)
              << ", mcm_ntotal_write="<< *(cl_start +2) <<", mcm_ntotal_read="<< *(cl_start +3)
              << ", edge_ntotal_write="<< *(cl_start +4) << ", edge_ntotal_read="<< *(cl_start +5)
              << ", debug_ntotal_write="<< *(cl_start +6) << ", debug_ntotal_read="<< *(cl_start +7)
              << ", debug_address_error32=" << *(cl_start +8) << ", debug_case_error="<<  *(cl_start +9)
              << ", num_issued_writes="<< *(cl_start +10) << ", num_received_acks="<< *(cl_start +11) << std::endl;
}

void MWMCrouchCSRRam::print_part_sorted_edges(volatile uint32_t* edges)
{
    for(uint32_t i = 0; i < 8*3; i++)
    {
        std::cout << edges[i] << " ";
    }
}

#endif
