
#ifdef FPGA_FLAG


#ifndef __MWM_CROUCH_CSR_RAM_IMPROVED_HPP__
#define __MWM_CROUCH_CSR_RAM_IMPROVED_HPP__

#include "../../mwmsolver/mwmsolver.hpp"
#include "../../util/util.hpp"
#include "../../util/bitvector.hpp"
#include "../../util/graph.hpp"

#include "../../util/centaur_config.hpp"


#define MWM_CROUCH_CSR_RAM_OP  26


struct MWM_CSR_RAM_AFU_CONFIG_IMPROVED {
    // CL #1
    union {
        uint64_t          qword0[8];       // make it a whole cacheline
        struct {
            void         * pPtrs;          // ptr_base_addr <= um_params[63:6];
            void         * pEdges;
            void         * pDestEdges;
            void         * pDestMCMBits;
            void         * pDestDebugEdges;
            void         * pDebug;
            uint32_t       num_ptrs         : 32;
            uint32_t       num_edges        : 32;
            uint32_t       edge_offset      : 32;

        };
    };
    // CL#2
};

/**
 * @brief The MWMCrouchCSRRamImproved class
 *
 * This class provides the host code for MWM crouch with custom CSR format.
 * Contrary to mwm_crouch_csr_ram, this class uses expects a list of edges as output format from the FPGA
 * This decreases the time to construct the final MWM approximation.
 */

class MWMCrouchCSRRamImproved : public MWMSolver
{
public:
   MWMCrouchCSRRamImproved(FPGA * fpga);
   ~MWMCrouchCSRRamImproved();

    void* solve(void* pinput, Timing* timer, RunParams* rp);

    double get_guarantee(RunParams* rp);

    std::string get_solver_name();

    double get_min_epsilon();

    double get_min_delta();

private:
    FPGA * myfpga;


    void print_part_sorted_edges(volatile uint32_t* edges);
    void print_bram_mcm_bit_receiver(volatile uint32_t* cl_start);
    void print_bram_mcm_bit_requester(volatile uint32_t* cl_start);
    void print_edge_processor(volatile uint32_t* cl_start);
    void print_edge_receiver(volatile uint32_t * cl_start);
    void print_edge_requester(volatile uint32_t* cl_start);
    void print_edge_writer(volatile uint32_t* cl_start);
    void print_mcm_bit_ack_receiver(volatile uint32_t* cl_start);
    void print_mcm_bit_receiver(volatile uint32_t* cl_start);
    void print_mcm_bit_requester(volatile uint32_t* cl_start);
    void print_mcm_bit_writer(volatile uint32_t* cl_start);
    void print_mwm_crouch_csr_ram(volatile uint32_t* cl_start);
    void print_pointer_receiver(volatile uint32_t* cl_start);
    void print_pointer_requester(volatile uint32_t* cl_start);
    void print_request_interface(volatile uint32_t* cl_start);
    void print_sorter(volatile uint32_t* cl_start);
    void print_state_controller(volatile uint32_t* p_cl_start);
    void print_write_interface(volatile uint32_t* cl_start);

};


#endif
#endif
