
#ifdef FPGA_FLAG


#ifndef __MWM_CROUCH_FPGA_HPP__
#define __MWM_CROUCH_FPGA_HPP__


#include "../../mwmsolver/mwmsolver.hpp"
#include "../../util/util.hpp"
#include "../../util/bitvector.hpp"
#include "../../util/graph.hpp"

#include "../../util/centaur_config.hpp"



#define MWM_CROUCH_BRAM_STEPWISE_OP  16
#define MWM_CROUCH_BRAM_FIFO_OP 17
#define COPY_OP 5

struct MWM_AFU_CONFIG {
    // CL #1
    union {
        uint64_t          qword0[8];       // make it a whole cacheline
        struct {
            void         * pSource;         /// 63:0
            void         * pDest;           /// 127:64
            uint32_t       num_cl;          //  191:128
        };
    };
    // CL#2
};


/**
 * @brief The MWMCrouchFPGA class
 * This class provides the host code for the naive MWM implementation of the FPGA. This version gets a single list
 * of edges from the FPGA and supports up to 32 MCMs computed on the FPGA.
 * Furthermore, all matching bits are stored on the FPGA (limits therefore the maximum number of vertices)
 * and there is no pipelining on the FPGA.
 */
class MWMCrouchFPGA : public MWMSolver
{
public:
   MWMCrouchFPGA(FPGA* fpga, int afu_id);
   ~MWMCrouchFPGA();

    void* solve(void* pinput, Timing* timer, RunParams* rp);

    double get_guarantee(RunParams* rp);

    std::string get_solver_name();

    double get_min_epsilon();

    double get_min_delta();

private:
    FPGA * myfpga;
    int afu_id;
};


#endif

#endif
