
#include "metisfileloader.hpp"

MetisFileLoader::MetisFileLoader()
{
    // empty on purpose
}


MetisFileLoader::~MetisFileLoader()
{
    // empty on purpose
}

void* MetisFileLoader::load_any(std::string path, int flag, uint32_t max_w)
{
    FileAbstraction fa;
    fa.open(path);

    if( ! Util::has_suffix(fa.file_name, MetisFileLoader::get_file_ending()) )
    {
        return nullptr;
    }


    if( ! fa.is_ok() )
    {
        std::cout << "Cannot open file at path: "<< path << std::endl;
        return nullptr;
    }


    std::string str;
    int temp;
    str = fa.get_line(&temp);
    std::vector<uint32_t> values = Util::split_whitespace_to_uint(str);

    if(values.size() <= 1)
    {
        std::cout << "Invalid Metis File: requires at least 2 or more params in first line (n, m, fmt, ncon)" << std::endl;
        return nullptr;
    }

    std:: cout << "Metis file with " << values.size() << " elements in the header " << std::endl;


    WEdgeList* wedge_list = (WEdgeList*) malloc(sizeof(WEdgeList));
    wedge_list->n = values[0];
    wedge_list->m = values[1];
    wedge_list->wedge = new std::vector<WEdge>;
    uint32_t max_weight = 0;

    EdgeList* edge_list= (EdgeList*) malloc(sizeof(EdgeList));
    edge_list->n = values[0];
    edge_list->m = values[1];
    edge_list->edge = new std::vector<Edge>();


    int32_t fmt = 0; // 0: graph has no weights for edge, nor vertex; 1: has edge weight; 10: has vertex weight; 11: has edge and vertex weight
    int32_t ncon = 0; // number of edge weights for this vertex

    if(values.size() >= 3)
    {
        fmt = values[2];
    }

    // with fmt 11 or 10, the ncon value is at least 1
    if(fmt == 11 || fmt == 10)
    {
        ncon = 1;
    }

    if(values.size() >= 4)
    {
        ncon = values[3];
    }

    std::cout << "Starting reading file "<< path << " with n="<< wedge_list->n <<", m="<< wedge_list->m<< ", fmt=" << fmt<<", ncon="<<ncon << std::endl;

    uint32_t curr_vertex_id = 1;
    uint32_t edge_counter = 0;

    std::set<Edge> set_map;


    while (edge_counter < wedge_list->m && edge_counter < edge_list->m)
    {
        int is_eof;
        str = fa.get_line(&is_eof);
        if(is_eof == 1)
        {
            break;
        }

        // ignore lines that start with %
        std::string ignore_value = "%";
        std::size_t found = str.find(ignore_value);
        if(found != std::string::npos)
        {
            continue;
        }

        std::vector<uint32_t> values = Util::split_whitespace_to_uint(str);


        // no edge weight
        if( fmt == 0 || fmt == 10)
        {
            for(auto it = ( values.begin() + ncon); it != values.end(); ++it)
            {
                WEdge we; we.u =curr_vertex_id; we.v = *it;
                Edge e; e.u = curr_vertex_id; e.v = *it;

                // ignore self-loops and edges, that we have already processed
                // also ignore edges that are already added
                if( *it <= curr_vertex_id || set_map.find(e) != set_map.end() || e.u == e.v)
                {
                    continue;
                }

                set_map.insert(e);

                if(flag == 1)
                {
                    we.weight =rand() % max_w;
                }
                else
                {
                    we.weight = 1;
                }

                if(flag == 0 || flag == 1)
                {
                    wedge_list->wedge->push_back(we);
                }
                else
                {
                    edge_list->edge->push_back(e);
                }

                max_weight = std::max( max_weight, (uint32_t)we.weight);
                edge_counter++;
            }
        }
        else
        {
            for(auto it = (values.begin() +  ncon); it != values.end(); )
            {
                WEdge we; we.u =curr_vertex_id; we.v = *it;
                Edge e; e.u = curr_vertex_id; e.v = *it;

                if( *it <= curr_vertex_id || set_map.find(e) != set_map.end())
                {
                    it = it+2;
                    continue;
                }
                set_map.insert(e);


                ++it;
                if(flag == 1)
                {
                    we.weight = rand() % max_w;
                }
                else
                {
                    we.weight = *it;
                }

                ++it;
                if(flag == 0 || flag == 1)
                {
                    wedge_list->wedge->push_back(we);
                }
                else
                {
                    edge_list->edge->push_back(e);
                }

                max_weight = std::max(max_weight, (uint32_t)we.weight);
                edge_counter++;
            }
        }

        curr_vertex_id++;
    }

    std::cout << "Ended reading file "<< path << std::endl;
    wedge_list->max_w = max_weight;

    edge_list->m = edge_list->edge->size();
    wedge_list->m = wedge_list->wedge->size();

    if(flag == 0 || flag == 1)
    {
        delete(edge_list->edge);
        free(edge_list);
        return wedge_list;
    }

    delete(wedge_list->wedge);
    free(wedge_list);
    return edge_list;
}

EdgeList* MetisFileLoader::uload(std::string path)
{
    return (EdgeList*) this->load_any(path, -1, 0);
}

WEdgeList* MetisFileLoader::load(std::string path, bool rand_weight, uint32_t max_w)
{
    return (WEdgeList*) this->load_any(path, (int)rand_weight, max_w);
}

std::string MetisFileLoader::get_file_ending()
{
    return ".graph";
}
