
#include "tsvfileloader.hpp"


TsvFileLoader::TsvFileLoader()
{
    // empty on purpose
}

TsvFileLoader::~TsvFileLoader()
{
    // empty on purpose
}



void* TsvFileLoader::load_any(std::string path, int flag, uint32_t max_w)
{
    FileAbstraction fa;
    fa.open(path);

    if( ! Util::has_suffix(fa.file_name, TsvFileLoader::get_file_ending()) )
    {
        return nullptr;
    }

    if( ! fa.is_ok() )
    {
        std::cout << "TsvFileLoader: Cannot open file at path: "<< path << std::endl;
        return nullptr;
    }

    uint32_t max_vertex_id = 0;
    uint32_t max_weight = 0;

    auto wedge_list = new std::vector<WEdge>();
    auto edge_list = new std::vector<Edge>();

    std::set<Edge> set_map;

    std::string str;

    while (true)
    {
        int is_eof;
        str = fa.get_line(&is_eof);
        if(is_eof == 1)
        {
            break;
        }

        // ignore lines that start with %
        std::string ignore_value = "%";
        std::size_t found = str.find(ignore_value);
        if(found != std::string::npos)
        {
            continue;
        }

        std::vector<uint32_t> values = Util::split_whitespace_to_uint(str);

        if(values.size() >= 2)
        {
            WEdge wedge;
            wedge.u = values[0];
            wedge.v = values[1];

            Edge edge;
            edge.u = wedge.u;
            edge.v = wedge.v;

            if(set_map.find(edge) != set_map.end())
            {
                continue;
            }

            if(edge.u == edge.v) // do not allow self-loops
            {
                continue;
            }

            if(flag == 1)
            {
                wedge.weight = rand() % max_w;
            }
            else
            {
                if(values.size() >= 3)
                {
                    wedge.weight = values[2];
                }
                else
                {
                    wedge.weight = 1;
                }
            }

            if(flag == 0 || flag == 1)
            {
                wedge_list->push_back(wedge);
            }
            else // flag == -1
            {
                edge_list->push_back( edge );
            }

            max_weight = std::max(max_weight, (uint32_t) wedge.weight);
            uint32_t temp = std::max(edge.u, edge.v);
            max_vertex_id = std::max( max_vertex_id, temp);
        }
    }

    WEdgeList* wresult = (WEdgeList*) malloc(sizeof(WEdgeList));
    EdgeList* result = (EdgeList*) malloc(sizeof(EdgeList));

    wresult->m = wedge_list->size();
    wresult->n = max_vertex_id;
    wresult->wedge = wedge_list;
    wresult->max_w = max_weight;

    result->m = edge_list->size();
    result->n = max_vertex_id;
    result->edge = edge_list;

    std::cout << "Ended reading file "<< path <<  std::endl;

    if(flag == 0 || flag == 1)
    {
        delete(edge_list);
        free(result);
        std::cout<< "sanitize the list"<< std::endl;
        Util::sanitize_wedge_list(wresult);
        std::cout << "File stats for weighted edge: m="<< wresult->m<<", n="<< wresult->n << std::endl;
        return wresult;
    }

    std::cout << "File stats: m="<< result->m<<", n="<< result->n << std::endl;
    delete(wedge_list);
    free(wresult);
    return result;
}


EdgeList* TsvFileLoader::uload(std::string path)
{
    return (EdgeList*) (this->load_any(path, -1, 0));
}


WEdgeList* TsvFileLoader::load(std::string path, bool rand_weights, uint32_t max_w)
{
    return (WEdgeList*) (this->load_any(path, (int)rand_weights, max_w) );
}

std::string TsvFileLoader::get_file_ending()
{
    return ".tsv";
}
