
#include <stdio.h>
#include <string>
#include <vector>
#include <stdlib.h>
#include <time.h>


#include "io/inputgenerator.hpp"

#include "mwmsolver/mwmserial.hpp"
#include "mwmsolver/mwmlemon.hpp"
#include "mwmsolver/mwmghaffariopenmp.hpp"
#include "mwmsolver/mwmcrouchserial.hpp"
#include "mwmsolver/mwmcrouchopenmpsimple.hpp"

#include "mcmsolver/mcmlemonsolver.hpp"
#include "mcmsolver/mcmserial.hpp"

#include "util/util.hpp"
#include "runconfig/runconfig.hpp"
#include "solver/solver.hpp"

#include "fpga/mwm/mwm_crouch_fpga.hpp"
#include "fpga/mwm/mwm_crouch_csr_ram.hpp"
#include "fpga/mwm/mwm_crouch_csr_ram_improved.hpp"


void print_usage();

/**
 * This main class can be used to run both CPU and FPGA algorithms.
 * However, during measurements, it has been more convenient to run them
 * separately.
 */
int main(int argc, char** argv)
{
    printf("Start of program\n");
    printf("Start of main\n");

    if(argc <= 1)
    {
        print_usage();
        return 0;
    }

    uint32_t seed = 1523728541; // time(NULL);
    std::cout << "seed="<< seed<<std::endl;
    srand (seed);
#ifdef PERFORMANCE
    std::cout << "PERFORMANCE: 1"  <<std::endl;
#else
    std::cout << "PERFORMANCE: 0"<< std::endl;
#endif

    std::vector<std::string> input_paths;

    int problem = atoi( argv[1] );

    int end = 0;
#ifdef FPGA_FLAG
    end = argc-1;
#else
    end = argc;
#endif

    for(int i = 2; i < end; i++)
    {
        printf("%s\n", argv[i]);
        std::string s(argv[i]);
        input_paths.push_back(s);
    }


    RunConfig rc(input_paths, seed);

    if(problem == 0)
    {

        auto mwmSerial = new MWMGhaffariSerial();
        auto mwmLemon = new MWMLemon();
        auto openMPv1 = new MWMGhaffariOpenMP();
        auto mwmSerialCrouch = new MWMCrouchSerial();
        auto mwmCrouchOpenMPSimple = new MWMCrouchSerialOpenMPSimple();

 #ifdef FPGA_FLAG
        std::cout << "Create new FPGA object" << std::endl;
        auto fpga = new FPGA();
        auto mwmCrouchCSRRam = new MWMCrouchCSRRamImproved(fpga);
        //MWMCrouchFPGA* mwmCrouchFPGA = new MWMCrouchFPGA(fpga, MWM_CROUCH_BRAM_FIFO_OP);
#endif

        rc.register_function(mwmSerial);
        rc.register_function(mwmSerialCrouch);

#ifdef FPGA_FLAG
        rc.register_function(mwmCrouchCSRRam);
#endif

        rc.register_function_exact(mwmLemon);
        rc.execute();

        delete(mwmSerialCrouch);
        delete(mwmSerial);
        delete(mwmLemon);
        delete(openMPv1);

#ifdef FPGA_FLAG
        sleep(10); // give all threads of ASE time to terminate
#endif
    }

    if(problem == 1)
    {
        auto mcmSerial = new MCMSerial();
        auto mcmLemon = new MCMLemonSolver();

        rc.register_function(mcmSerial);
        rc.register_function(mcmLemon);
        rc.register_function_exact(mcmLemon);
        rc.execute();

        delete(mcmSerial);
        delete(mcmLemon);
    }


    puts("End of main");

    sleep(10); // give all threads of ASE time to terminate
    puts("End of program");
    return 0;
}

void print_usage()
{
    puts("USAGE");
    puts("./main_run PATH_TO_UNITTEST_OR_PERFORMANCE_FILE");
}
