
// the flag is set using the cmake-file
// #define PERFORMANCE 1


#include <stdio.h>
#include <string>
#include <vector>
#include <stdlib.h>
#include <time.h>


#include "io/inputgenerator.hpp"

#include "mwmsolver/mwmserial.hpp"
#include "mwmsolver/mwmlemon.hpp"
#include "mwmsolver/mwmghaffariopenmp.hpp"
#include "mwmsolver/mwmcrouchserial.hpp"
#include "mwmsolver/mwmcrouchopenmpsimple.hpp"

#include "mcmsolver/mcmlemonsolver.hpp"
#include "mcmsolver/mcmserial.hpp"

#include "util/util.hpp"
#include "runconfig/runconfig.hpp"
#include "solver/solver.hpp"

#include "fpga/mwm/mwm_crouch_fpga.hpp"
#include "fpga/mwm/mwm_crouch_csr_ram.hpp"
#include "fpga/mwm/mwm_crouch_csr_ram_improved.hpp"

void print_usage();

/**
 * This main method is used to run CPU-only algorithms, for both
 * performance measurements and testing the correctness by comparing the output
 * to the precise results of the Lemon Graph Library.
 */
int main(int argc, char** argv)
{
    puts("Start of program: main_run_cpu\n");

    if(argc <= 1)
    {
        print_usage();
        return 0;
    }

    uint32_t seed = 1523728541;
    std::cout << "seed="<< seed<<std::endl;
    srand (seed);

    std::vector<std::string> input_paths;

    int end = 0;
    end = argc;

    puts("\nStart paths to search for files:\n");
    for(int i = 1; i < end; i++)
    {
        printf("%s\n", argv[i]);
        std::string s(argv[i]);
        input_paths.push_back(s);
    }
    puts("End paths to search for files\n\n");

    RunConfig rc(input_paths, seed);

    auto mwmSerial = new MWMGhaffariSerial();
    auto mwmLemon = new MWMLemon();
    auto openMPv1 = new MWMGhaffariOpenMP();
    auto mwmSerialCrouch = new MWMCrouchSerial();
    auto mwmCrouchOpenMPSimple = new MWMCrouchSerialOpenMPSimple();


    rc.register_function(mwmSerial);
    rc.register_function(mwmSerialCrouch);
    rc.register_function(mwmCrouchOpenMPSimple);

    rc.register_function_exact(mwmLemon);
    rc.execute();

    delete(mwmSerial);
    delete(mwmLemon);
    delete(openMPv1);
    delete(mwmSerialCrouch);
    delete(mwmCrouchOpenMPSimple);

    puts("End of main");
    puts("End of program");
    return 0;
}

void print_usage()
{
    puts("USAGE");
    puts("./main_run_cpu PATH_TO_UNITTEST_OR_PERFORMANCE_FILE");
}
