
#include <stdio.h>
#include <string>
#include <vector>
#include <stdlib.h>
#include <time.h>


#include "io/inputgenerator.hpp"

#include "mwmsolver/mwmserial.hpp"
#include "mwmsolver/mwmlemon.hpp"
#include "mwmsolver/mwmghaffariopenmp.hpp"
#include "mwmsolver/mwmcrouchserial.hpp"
#include "mwmsolver/mwmcrouchopenmpsimple.hpp"

#include "mcmsolver/mcmlemonsolver.hpp"
#include "mcmsolver/mcmserial.hpp"

#include "util/util.hpp"
#include "runconfig/runconfig.hpp"
#include "solver/solver.hpp"

#include "fpga/mwm/mwm_crouch_fpga.hpp"
#include "fpga/mwm/mwm_crouch_csr_ram.hpp"
#include "fpga/mwm/mwm_crouch_csr_ram_improved.hpp"

void print_usage();

/**
 * This main method starts the FPGA implementation with custom CSR format.
 * Also, it allows to compare the result of the FPGA implementation to
 * Lemon.
 */
int main(int argc, char** argv)
{
    puts("Start of program main_run_fpga_improved");

    if(argc <= 1)
    {
        print_usage();
        return 0;
    }

    uint32_t seed = 1523728541;
    std::cout << "seed="<< seed<<std::endl;
    srand (seed);

    std::vector<std::string> input_paths;

    int end = 0;
    end = argc;

    for(int i = 1; i < end; i++)
    {
        printf("%s\n", argv[i]);
        std::string s(argv[i]);
        input_paths.push_back(s);
    }

    RunConfig rc(input_paths, seed);
    auto mwmLemon = new MWMLemon();


#ifdef FPGA_FLAG
    std::cout << "Create new FPGA object" << std::endl;
    auto fpga = new FPGA();
    auto mwmCrouchCSRRamImproved = new MWMCrouchCSRRamImproved(fpga);
#endif


#ifdef FPGA_FLAG
    rc.register_function(mwmCrouchCSRRamImproved);
#endif

    rc.register_function_exact(mwmLemon);
    rc.execute();

    delete(mwmLemon);


#ifdef FPGA_FLAG
    sleep(10); // give all threads of ASE time to terminate
    delete(mwmCrouchCSRRamImproved);
#endif


    sleep(10); // give all threads of ASE time to terminate
    std::cout << "End of program" << std::endl;
    return 0;
}

void print_usage()
{
    puts("USAGE");
    puts("./main_run_fpga_improved PATH_TO_UNITTEST_OR_PERFORMANCE_FILE");
}
