
#include "mcmlemonsolver.hpp"



MCMLemonSolver::MCMLemonSolver()
{
    // empty on purpose
}

MCMLemonSolver::~MCMLemonSolver()
{
    // empty on purpose
}

void* MCMLemonSolver::solve(void* pinput, Timing* timer, RunParams* rp)
{
    EdgeList* input = (EdgeList*)pinput;

    uint32_t prefix_timing_id = timer->startTiming();

    // create the required objects
    lemon::ListGraph g;

    Util::edge_list_to_lemon_graph(input, &g);

    timer->endTiming(prefix_timing_id, "prefix", false);
    uint32_t algo_timing_id = timer->startTiming();

    lemon::MaxMatching<lemon::ListGraph> mcm(g);
    mcm.run();

    timer->endTiming(algo_timing_id, "full_algo", false);
    uint32_t suffix_timing_id = timer->startTiming();

    auto& mm = mcm.matchingMap();
    auto edgeList = new std::vector<Edge>( ) ;
    std::vector <bool> bs(input->n + 1);

    for (lemon::ListGraph::NodeIt n(g); n != lemon::INVALID; ++n)
    {
        if( (mm)[n] != lemon::INVALID )
        {
            lemon::ListGraph::Edge e = (mm)[n];

            lemon::ListGraph::Node n1 = g.u(e);
            lemon::ListGraph::Node n2 = g.v(e);
            int id1 = g.id(n1);
            int id2 = g.id(n2);

            if(id1 != id2 && !bs[id1] && !bs[id2] )
            {
                Edge we;
                we.u = id1;
                we.v = id2;
                edgeList->push_back(we);
                bs[id1] = true;
                bs[id2] = true;
            }
        }
    }


    EdgeList* result = (EdgeList*) malloc(sizeof(EdgeList));
    result->n = input->n;
    result->edge = edgeList;
    result->m = edgeList->size();

    timer->endTiming(suffix_timing_id, "suffix", false);

    return result;
}

double MCMLemonSolver::get_guarantee(RunParams * rp)
{
    return 1.0;
}

std::string MCMLemonSolver::get_solver_name()
{
    return "mcm_lemon";
}
