
#include "mcmsolver.hpp"


int MCMSolver:: get_input_type()
{
    return Solver::INPUT_TYPE_UNWEIGHTED;
}


std::string MCMSolver::get_problem_name()
{
    return "mcm";
}


void MCMSolver::delete_output(void* poutput)
{
    EdgeList* output = (EdgeList*)poutput;
    delete(output->edge);
    free(output);
}


void MCMSolver::delete_input(void* pinput)
{
    EdgeList* input = (EdgeList*)pinput;
    delete(input->edge);
    free(input);
}


bool MCMSolver::is_result_correct(void* ptruth, void* pcomputed, RunParams* rp)
{
    EdgeList* truth = (EdgeList*)ptruth;
    EdgeList* computed = (EdgeList*) pcomputed;

    uint64_t cardinality_exact = MCMSolver::get_matching_cardinality(truth);
    uint64_t cardinality_computed = MCMSolver::get_matching_cardinality(computed);
    double cardinality_ratio = ( (double)cardinality_exact/ (double)cardinality_computed);
    double guarantee = this->get_guarantee(rp);

    std::cout<< "cardinality_exact="<<cardinality_exact<< std::endl;
    std::cout << "cardinality_computed="<< cardinality_computed << std::endl;
    std::cout << "cardinality_ratio="<< cardinality_ratio << std::endl;
    std::cout << "guarantee="<< guarantee << std::endl;

    return !(cardinality_ratio > guarantee);
}


uint64_t MCMSolver::get_matching_cardinality(EdgeList* matching)
{
    return matching->edge->size();
}


