
#ifdef FPGA_FLAG


#ifndef __MICRO_BENCH_HPP__
#define __MICRO_BENCH_HPP__


#include "../mwmsolver/mwmsolver.hpp"
#include "../util/util.hpp"
#include "../util/bitvector.hpp"
#include "../util/graph.hpp"

#include "../util/centaur_config.hpp"


#define MICROBENCH_OP  22

struct MICROBENCH_AFU_CONFIG {
    // CL #1
    union {
        uint64_t          qword0[8];       // make it a whole cacheline
        struct {
            void         * pSource;         /// 63:0
            void         * pDest;           /// 127:64
            uint32_t       num_cl;          //  159:128
            uint32_t       nitems_latency;  // 191:160
            uint32_t       timeout;         // 223:192
        };
    };
    // CL#2
};


struct Latency
{
    std::vector<uint32_t> tput_cycles_read;
    std::vector<uint32_t> tput_cl_read;
    std::vector<uint32_t> tput_cycles_write;
    std::vector<uint32_t> tput_cl_write;
    std::vector<uint32_t> tput_cycles_read_write;
    std::vector<uint32_t> tput_cl_read_write;

    std::vector<uint32_t> latency_cycles_read;
    std::vector<uint32_t> latency_cl_read;
    std::vector<uint32_t> latency_cycles_write;
    std::vector<uint32_t> latency_cl_write;
    std::vector<uint32_t> latency_cycles_read_write;
    std::vector<uint32_t> latency_cl_read_write;

    std::vector<uint32_t> tput_rand_cycles_read;
    std::vector<uint32_t> tput_rand_cl_read;
    std::vector<uint32_t> tput_rand_cycles_write;
    std::vector<uint32_t> tput_rand_cl_write;
    std::vector<uint32_t> tput_rand_cycles_read_write;
    std::vector<uint32_t> tput_rand_cl_read_write;

    std::vector<uint32_t> latency_rand_cycles_read;
    std::vector<uint32_t> latency_rand_cl_read;
    std::vector<uint32_t> latency_rand_cycles_write;
    std::vector<uint32_t> latency_rand_cl_write;
    std::vector<uint32_t> latency_rand_cycles_read_write;
    std::vector<uint32_t> latency_rand_cl_read_write;
};
typedef struct Latency Latency;


/**
 * @brief The MicroBench class
 *
 * This class provides an interface to the microbench FPGA function.
 * It can be used to determine basic latency and throughput values.
 * Unfortunately, the implementation is not finished and the values measured
 * must be handled with caution.
 * The implementation of this class has not been used in the paper.
 */
class MicroBench
{
public:
   MicroBench();
   ~MicroBench();

    void run(uint32_t num_cache_lines, uint32_t nitems_latency, uint32_t timeout, Latency *l );

    void compute_results(Latency *l);

private:
    FPGA * myfpga;
    int afu_id;

    void compute_mean_std_cycles(std::vector<uint32_t> values, double* mean, double* std);

    void compute_mean_std_bytes_per_cycles(std::vector<uint32_t> values, uint32_t bytes, double* mean, double* std);

    void compute_mean_std(std::vector<double> values, double* mean, double* std);

    void compute_mean_std_latency(std::vector<uint32_t> values, uint32_t trials, double* mean, double* std);

};


#endif

#endif

