
#include "mwmcrouchopenmpsimple.hpp"

MWMCrouchSerialOpenMPSimple::MWMCrouchSerialOpenMPSimple()
{
    // empty on purpose
}

MWMCrouchSerialOpenMPSimple::~MWMCrouchSerialOpenMPSimple()
{
    // empty on purpose
}

void* MWMCrouchSerialOpenMPSimple::solve(void* pinput, Timing* timer, RunParams* rp)
{
    WEdgeList* input = (WEdgeList*) pinput;
    const int n_threads = rp->nthreads;

    uint32_t prefix_timer = timer->startTiming();
    uint32_t n_levels = rp->num_levels;

    std::vector<std::vector<WEdge>> wedges;
    for(int i = 0; i < n_levels; i++)
    {
        std::vector<WEdge> temp;
        wedges.push_back(temp);
    }

    uint32_t* level_weights = (uint32_t*)malloc(sizeof(uint32_t) * n_levels);
    const double level_weight_const = 1+rp->epsilon;
    double level_weight = 1;
    for(int i = 0; i < n_levels; i++)
    {
        level_weights[i] = std::round(level_weight);
        level_weight = level_weight * level_weight_const;
#ifndef PERFORMANCE
        std::cout << "level["<<i<<"]="<<level_weights[i]<<std::endl;
#endif
    }

    uint8_t* mcm_bits = (uint8_t*) calloc(sizeof(uint8_t), n_levels * input->n );
    timer->endTimingSetup(prefix_timer);
    uint32_t processing_loop_timer = timer->startTiming();

#pragma omp parallel num_threads(n_threads) default(none) shared(input, level_weights, wedges, n_levels, mcm_bits)
        {

#pragma omp for
        // for each level
        for(int32_t i = n_levels -1; i >= 0; i--)
        {
            // for each edge
            for(uint32_t j = 0; j < input->wedge->size(); j++)
            {
                // get the edge and compute
                WEdge e = input->wedge->at(j);

                if( e.weight >= level_weights[i] )
                {
                    if( mcm_bits[i * input->n + e.u ] == 0 && mcm_bits[i* input->n + e.v ] == 0)
                    {
                        mcm_bits[i * input->n + e.u ] = 1;
                        mcm_bits[i* input->n + e.v ] = 1;
                        wedges[i].push_back(e);
                    }
                }
            }
        }
    }
    delete(mcm_bits);
    free(level_weights);

    timer->endTimingProcessingLoop(processing_loop_timer);
    uint32_t merge_loop_timer = timer->startTiming();

    auto bv = new BitVector(input->n);
    int offset = 1;

#ifndef PERFORMANCE
    std::ofstream newFile("mwmcrouchopenmpsimple.out.txt");
#endif
    std::vector<WEdge>* edgeList = new std::vector<WEdge>( ) ;
    for(int32_t j = wedges.size() -1; j >= 0 ; j--)
    {
        for(uint32_t k = 0; k < wedges[j].size(); k++)
        {
            WEdge e = wedges[j][k];
            if( bv->get(e.v) == 0 && bv->get(e.u) == 0)
            {

#ifndef PERFORMANCE
                newFile << "e.u="<<e.u<<", e.v="<< e.v<<", e.weight="<<e.weight<< ", e.max_pos="<< j + offset << std::endl;
#endif
                bv->set_true(e.v);
                bv->set_true(e.u);
                edgeList->push_back(e);
            }
        }
    }
#ifndef PERFORMANCE
    newFile.close();
#endif

    delete(bv);

    timer->endTimingMergeLoop(merge_loop_timer);
    uint32_t suffix_timer = timer->startTiming();

    WEdgeList* result = (WEdgeList*) malloc(sizeof(WEdgeList));
    result->n = input->n;
    result->wedge = edgeList;
    result->m = edgeList->size();

    timer->endTiming(suffix_timer, "suffix", false);
    return result;
}

double MWMCrouchSerialOpenMPSimple::get_guarantee(RunParams* rp)
{
    return rp->epsilon + 4.0f;

}

std::string MWMCrouchSerialOpenMPSimple::get_solver_name()
{
    return "mwm_crouch_openmp_simple";
}

double MWMCrouchSerialOpenMPSimple::get_min_epsilon()
{
    return 0.8;
}

double MWMCrouchSerialOpenMPSimple::get_min_delta()
{
    return 0.0;
}

