
#include "mwmcrouchserial.hpp"

MWMCrouchSerial::MWMCrouchSerial()
{
    // empty on purpose
}

MWMCrouchSerial::~MWMCrouchSerial()
{
    // empty on purpose
}

void* MWMCrouchSerial::solve(void* pinput, Timing* timer, RunParams* rp)
{
    WEdgeList* input = (WEdgeList*) pinput;

    uint32_t prefix_timer = timer->startTiming();
    const uint32_t n_levels = rp->num_levels;
    auto *bv = new BitVector ( (uint32_t)input->n * n_levels);


    std::vector<std::vector<WEdge>> wedges;
    for(int i = 0; i < n_levels; i++)
    {
        std::vector<WEdge> temp;
        wedges.push_back(temp);
    }

    uint32_t* level_weights = (uint32_t*)malloc(sizeof(uint32_t) * n_levels);
    const double level_weight_const = 1 + rp->epsilon;
    double level_weight = 1;
    for(int i = 0; i < n_levels; i++)
    {
        level_weights[i] = std::round(level_weight); // std::ceil
        level_weight = level_weight * level_weight_const;
#ifndef PERFORMANCE
        std::cout << "level["<<i<<"]="<<level_weights[i]<<std::endl;
#endif
    }

    timer->endTimingSetup(prefix_timer);
    uint32_t processing_loop_timer = timer->startTiming();

    // for each edge
    for(uint32_t j = 0; j < input->wedge->size(); j++)
    {
        // get the edge and compute
        WEdge e = input->wedge->at(j);

        uint32_t level_index = 0;

        // go over the levels and check, if we can add an edge
        // note that an idea was to go through the levels, starting with the highest, and as we added an edge,
        // we would not have to add it into the lower ones, so we could break. But then the output might not be that sparse anymore
        bool has_added = false;
        for(int32_t i = n_levels -1; i >= 0; i--)
        {
            if( e.weight >= level_weights[i] )
            {
                if( bv->get(level_index + (e.u-1)*n_levels ) == 0 && bv->get(level_index + (e.v-1)*n_levels) == 0)
                {
                    bv->set_true(level_index + (e.u-1)*n_levels);
                    bv->set_true(level_index + (e.v-1)*n_levels);
                    if( !has_added )
                    {
                        wedges[i].push_back(e);
                        has_added = true;
                    }
                }
            }
            level_index= level_index + 1;
        }
    }
    delete(bv);
    free(level_weights);

    timer->endTimingProcessingLoop(processing_loop_timer);
    uint32_t merge_loop_timer = timer->startTiming();

    bv = new BitVector(input->n);


#ifndef PERFORMANCE
    int offset = 1;
    std::ofstream newFile("mwmcrouchserial.out.txt");
#endif
    auto edgeList = new std::vector<WEdge>( ) ;
    for(int32_t j = wedges.size() -1; j >= 0 ; j--)
    {
        for(uint32_t k = 0; k < wedges[j].size(); k++)
        {
            WEdge e = wedges[j][k];
            if( bv->get(e.v) == 0 && bv->get(e.u) == 0)
            {
#ifndef PERFORMANCE
                newFile << "e.u="<<e.u<<", e.v="<< e.v<<", e.weight="<<e.weight<< ", e.max_pos="<< j + offset << std::endl;
#endif
                bv->set_true(e.v);
                bv->set_true(e.u);
                edgeList->push_back(e);
            }
        }
    }
#ifndef PERFORMANCE
    newFile.close();
#endif

    delete(bv);

    timer->endTimingMergeLoop(merge_loop_timer);
    uint32_t suffix_timer = timer->startTiming();

    WEdgeList* result = (WEdgeList*) malloc(sizeof(WEdgeList));
    result->n = input->n;
    result->wedge = edgeList;
    result->m = edgeList->size();

    timer->endTimingTeardown(suffix_timer);
    return result;
}

double MWMCrouchSerial::get_guarantee(RunParams* rp)
{
    return rp->epsilon + 4.0f;

}

std::string MWMCrouchSerial::get_solver_name()
{
    return "mwm_crouch_serial";
}

double MWMCrouchSerial::get_min_epsilon()
{
    return 0.8;
}

double MWMCrouchSerial::get_min_delta()
{
    return 0.0;
}

