
#include "mwmlemon.hpp"

void* MWMLemon::solve(void*  pinput, Timing* timer, RunParams* rp)
{
    WEdgeList* input = (WEdgeList*) pinput;
    // create the required objects
    lemon::ListGraph g;
    lemon::ListGraph::EdgeMap<int> cost (g);

    Util::wedge_list_to_lemon_graph(input, &g, &cost);

    lemon::MaxWeightedMatching<lemon::ListGraph> mwm(g, cost);

    mwm.init();
    mwm.run();

    std::vector <bool> bs(input->n + 1);

    auto& mm = mwm.matchingMap();
    auto edgeList = new std::vector<WEdge>( ) ;

    for (lemon::ListGraph::NodeIt n(g); n != lemon::INVALID; ++n)
    {
        if( (mm)[n] != lemon::INVALID )
        {
            lemon::ListGraph::Edge e = (mm)[n];

            lemon::ListGraph::Node n1 = g.u(e);
            lemon::ListGraph::Node n2 = g.v(e);

            int weight = cost[e];
            int id1 = g.id(n1);
            int id2 = g.id(n2);

            if(id1 != id2 && !bs[id1] && !bs[id2] )
            {
                WEdge we;
                we.u = id1;
                we.v = id2;
                we.weight = weight;
                edgeList->push_back(we);
                bs[id1] = true;
                bs[id2] = true;
            }
        }
    }

    WEdgeList* result = (WEdgeList*) malloc(sizeof(WEdgeList));
    result->n = input->n;
    result->wedge = edgeList;
    result->m = edgeList->size();

    return result;
}


std::string MWMLemon::get_solver_name()
{
    return "mwm_lemon_solver_exact";
}

double MWMLemon::get_guarantee(RunParams* rp)
{
    return 1.0f;
}
