
#include "mwmsolver.hpp"


int MWMSolver::get_input_type()
{
    return Solver::INPUT_TYPE_WEIGHTED;
}


std::string MWMSolver::get_problem_name()
{
    return "mwm";
}

void MWMSolver::delete_output(void* poutput)
{
    if(poutput != nullptr)
    {
        WEdgeList* output = (WEdgeList*)poutput;
        delete(output->wedge);
        free(output);
    }
}

void MWMSolver::delete_input(void* pinput)
{
    if(pinput != nullptr)
    {
        WEdgeList* input = (WEdgeList*) pinput;
        delete(input->wedge);
        free(input);
    }
}


uint64_t MWMSolver::get_matching_weight(WEdgeList* matching)
{
    uint64_t result = 0;
    for(uint32_t i = 0; i < matching->m; i++)
    {
        result = result + matching->wedge->at(i).weight;
    }
    return result;
}


uint64_t MWMSolver::get_matching_cardinality(WEdgeList* matching)
{
    return matching->m;
}


bool MWMSolver::is_result_correct(void* ptruth, void* pcomputed, RunParams* rp)
{
    if(ptruth == nullptr || pcomputed == nullptr)
    {
        return false;
    }

    WEdgeList* truth = (WEdgeList*) ptruth;
    WEdgeList* computed = (WEdgeList*)pcomputed;

    uint64_t cardinality_computed = MWMSolver::get_matching_cardinality(computed);
    uint64_t weight_computed = MWMSolver:: get_matching_weight(computed);

    uint64_t cardinality_exact = MWMSolver::get_matching_cardinality(truth);
    uint64_t weight_exact = MWMSolver::get_matching_weight(truth);

    double cardinality_ratio = 0;
    double weight_ratio = 0;
    if( cardinality_computed != 0)
    {
        cardinality_ratio = ( (double)cardinality_exact/ (double)cardinality_computed);
    }

    if( weight_computed != 0)
    {
        weight_ratio = ( (double)weight_exact/(double)weight_computed );
    }

    double guarantee = this->get_guarantee(rp);

    std::cout << "cardinality_exact = "<<cardinality_exact << ", weight_exact="<< weight_exact << std::endl;
    std::cout << "cardinality_computed="<< cardinality_computed << ", weight_computed="<<weight_computed<< std::endl;
    std::cout << "guarantee="<< guarantee << std::endl; //  << "edge_ratio="<< edge_ratio << ",
    std::cout << "cardinality_ratio="<< cardinality_ratio << ", weight_ratio="<< weight_ratio << std::endl;

    if(cardinality_computed == 0 && cardinality_exact != 0 && weight_computed == 0 && weight_exact != 0)
    {
        return false;
    }

    if( cardinality_computed == cardinality_exact && weight_computed == weight_exact)
    {
        return true;
    }

    if(weight_computed > weight_exact)
    {
        return false;
    }

    if(weight_ratio > guarantee)
    {
        return false;
    }

    return true;
}
