
#include "commonconfig.hpp"


CommonConfig::CommonConfig(std::string path, uint32_t seed)
{
    this->seed = seed;

    std::ifstream ifs(path);
    rapidjson::IStreamWrapper isw(ifs);
    rapidjson::Document d;
    d.ParseStream(isw);

#ifdef HWAFU

    if( d.HasMember("path_input_harp"))
    {
        rapidjson::Value& p = d["path_input_harp"];
        this->file_path = p.GetString();
    }
    else
    {
        if(d.HasMember("path_input"))
        {
            rapidjson::Value& p = d["path_input"];
            this->file_path = p.GetString();
            std::cout << "use default path_input!"<< std::endl;
        }
        else
        {
            throw std::invalid_argument("Did not find member path_input in file "+ file_path);
        }
    }
#else

    if(d.HasMember("path_input"))
    {
        rapidjson::Value& p = d["path_input"];
        this->file_path = p.GetString();
    }
    else
    {
        throw std::invalid_argument("Did not find member path_input in file "+ file_path);
    }
#endif

    if(d.HasMember("name"))
    {
        rapidjson::Value& name = d["name"];
        this->experiment_name = name.GetString();
    }
    else
    {
        throw std::invalid_argument("Did not find member name in file "+ file_path);
    }


    if(! d.HasMember("configs"))
    {
        throw std::invalid_argument("Did not find member configs in file "+ file_path);
    }
    rapidjson::Value& configs = d["configs"];

    std::cout << "file_path="<< this->file_path<< ", name="<< this->experiment_name<< std::endl;
    for(int i = 0; i < configs.GetArray().Size(); i++)
    {
        RunParams rp;
        if(configs.GetArray()[i].HasMember("m") && configs.GetArray()[i].HasMember("n") && configs.GetArray()[i].HasMember("epsilon"))
        {
            rp.m = configs.GetArray()[i]["m"].GetInt();
            rp.n = configs.GetArray()[i]["n"].GetInt();
            rp.epsilon = configs.GetArray()[i]["epsilon"].GetDouble();
            rp.rand_weights = false;
            rp.rep = 1;
            rp.nthreads = 1;
            rp.delta = 0.05;
            rp.num_levels = 8;
            rp.k = 0;
            rp.max_w = 100;


            if(configs.GetArray()[i].HasMember("rand_weights"))
            {
                rp.rand_weights = configs.GetArray()[i]["rand_weights"].GetBool();
            }

            if( configs.GetArray()[i].HasMember("rep") )
            {
                rp.rep = configs.GetArray()[i]["rep"].GetInt();
            }

            if(configs.GetArray()[i].HasMember("nthreads"))
            {
                rp.nthreads = configs.GetArray()[i]["nthreads"].GetInt();
            }

            if(configs.GetArray()[i].HasMember("delta"))
            {
                rp.delta = configs.GetArray()[i]["delta"].GetDouble();
            }

            if( configs.GetArray()[i].HasMember("num_levels"))
            {
                rp.num_levels = configs.GetArray()[i]["num_levels"].GetInt();
            }

            if( configs.GetArray()[i].HasMember("k"))
            {
                rp.k = configs.GetArray()[i]["k"].GetInt();
            }

            if( configs.GetArray()[i].HasMember("max_w"))
            {
                rp.max_w = configs.GetArray()[i]["max_w"].GetInt();
            }
            if( configs.GetArray()[i].HasMember("max_w"))
            {
                rp.max_w = configs.GetArray()[i]["max_w"].GetInt();
            }



            std::cout << "m="<< rp.m << ", n="<< rp.n << ", rep="<< rp.rep
                     << ", rand_weights="<< rp.rand_weights <<", epsilon="<< rp.epsilon <<", delta="<< rp.delta
                     << ", nthreads="<< rp.nthreads << ", num_levels="<< rp.num_levels<< ", k="<< rp.k
                     << ", max_w="<< rp.max_w << std::endl;
            this->run_params.push_back(rp);

        }
        else
        {
            throw std::invalid_argument("Did not find some member in file "+ file_path);
        }


    }
}


CommonConfig::~CommonConfig()
{
    // empty on purpose
}

void CommonConfig::register_function(Solver * solver)
{
    this->solvers.push_back(solver);
}

