
#include "runconfig.hpp"


RunConfig::RunConfig(std::string path, uint32_t seed)
{
    this->paths.push_back(path);
    this->seed = seed;
}

RunConfig::RunConfig(std::vector< std::string> ppaths, uint32_t seed )
{
    this->paths.insert(this->paths.end(), ppaths.begin(), ppaths.end());
    this->seed = seed;
}

RunConfig::~RunConfig()
{
    // empty on purpose
}

void RunConfig::execute()
{

    for(auto it = this->paths.begin(); it != this->paths.end(); ++it)
    {
        std::string path = *it;
        if(Util::is_dir(path.c_str()))
        {
            DIR* dirp = opendir(path.c_str());
            struct dirent *dp;
            while ((dp = readdir(dirp)) != NULL)
            {
                std::string file_name( dp->d_name);
                std::string full_path;
                if( Util::has_suffix(path, "/") )
                {
                    full_path = path+file_name;
                }
                else
                {
                    full_path = path+"/"+file_name;
                }

                this->execute_file( full_path );
            }
            (void)closedir(dirp);
        }
        else
        {
            if(Util::is_file(path.c_str()))
            {
                this->execute_file(path);
            }
            else
            {
                std::cout << "Path is not a directory, nor a file: "<< path << std::endl;
            }
        }
    }
}

void RunConfig::execute_file(std::string file_path)
{
    // file for performance
    if(Util::has_suffix( file_path, PerformanceConfig::FILE_EXTENSION ))
    {
            PerformanceConfig pc(file_path, this->seed);

            std::cout << "Run performance: " << file_path << std::endl;
            for(int i = 0; i < this->solvers.size(); i++)
            {
                pc.register_function(solvers[i]);
            }

            pc.execute();
    }
    else
    {
        if(Util::has_suffix(file_path, UnitTestConfig::FILE_EXTENSION))
        {
            UnitTestConfig utc(file_path, this->seed);

                std::cout << "Run unittest: " << file_path << std::endl;
                for(int i = 0; i < this->solvers.size(); i++)
                {
                    utc.register_function(solvers[i]);
                }
                utc.register_function_exact(this->solver_exact);
                utc.execute();
        }
        else
        {
            std::cout <<"Unknown file extension: " << file_path<< std::endl;
        }
    }
}

void RunConfig::register_function(Solver *solver)
{
    solvers.push_back(solver);
}

void RunConfig::register_function_exact(Solver *solver)
{
    this->solver_exact = solver;
}
