
#include "unittestconfig.hpp"

const std::string UnitTestConfig::FILE_EXTENSION = ".test";

UnitTestConfig::UnitTestConfig(std::string file_path, uint32_t seed) : CommonConfig(file_path, seed)
{
    // empty on purpose
}

UnitTestConfig::~UnitTestConfig()
{
    // empty on purpose
}

void UnitTestConfig::register_function_exact(Solver* solver)
{
    this->exact_solver = solver;
}


void UnitTestConfig::execute()
{
    std::cout << "----------" << std::endl;

    InputGenerator ig;

    uint32_t fail_count = 0;
    uint32_t pass_count = 0;
    uint32_t runs = 0;

    // iterate over all different run-configurations
    for(auto itp = this->run_params.begin(); itp != this->run_params.end(); ++itp)
    {
        void* list = nullptr;
        if(this->exact_solver->get_input_type() == Solver::INPUT_TYPE_DYNAMIC_UNWEIGHTED || this->exact_solver->get_input_type() == Solver::INPUT_TYPE_UNWEIGHTED)
        {
            list = ig.uload(this->file_path);
            if(list == nullptr)
            {
                return;
            }

            EdgeList* l = (EdgeList*) list;
            Util::print_edge_list( l, 10);
        }
        else
        {
            uint32_t k = (*itp).k;
            list = ig.load(this->file_path, (*itp).rand_weights, (*itp).max_w);

            if(list == nullptr)
            {
                return;
            }

            if( k >= 1)
            {
                WEdgeList* ltemp = (WEdgeList*) list;
                WEdgeList* l = Util::convert_wedgelist_to_k_merged_wedgelist(ltemp, k);
                list = (void*)l;

                Util::print_edge_list(l, 0, k);

                delete(ltemp->wedge);
                ltemp->wedge = nullptr;
                free(ltemp);
                ltemp = nullptr;
            }

        }

        RunParams rp = *itp;
        Timing t( &rp, this->file_path, " ");

        std::cout<< "Compute exact solution of "<< this->exact_solver->get_problem_name() << std::endl;
        void* exact = this->exact_solver->solve(list, &t, &rp);

        for(auto it = this->solvers.begin(); it != this->solvers.end(); ++it)
        {
            double orig_epsilon = rp.epsilon;
            double orig_delta = rp.delta;

            std::cout << "solver="<< (*it)->get_solver_name() << ", nthreads="<< rp.nthreads<< ", epsilon="<<rp.epsilon <<  ", delta="<<rp.delta << std::endl;
            void* result =  (*it)->solve( list, &t, &rp);

            bool is_correct = (*it)->is_result_correct(exact, result, &rp);

            if( !is_correct)
            {
                std::cout << Color::FG_RED << "---FAILED---" << Color::FG_DEFAULT<< std::endl;
                fail_count ++;
            }
            else
            {
                std::cout << Color::FG_GREEN << "---PASSED---"<<Color::FG_DEFAULT << std::endl;
                pass_count++;
            }
            runs++;

            rp.epsilon = orig_epsilon;
            rp.delta = orig_delta;

            (*it)->delete_output(result);
        }

        this->exact_solver->delete_output(exact);
        this->exact_solver->delete_input(list);
    }

    std::cout << "----------" << std::endl;
    std::cout << "nruns="<< runs<<", failed="<< fail_count <<", passed="<< pass_count<< std::endl;
}
