
#ifndef __SOLVER_HPP__
#define __SOLVER_HPP__

#include <string.h>

#include "../util/graph.hpp"
#include "../util/timing.hpp"

/**
 * @brief The Solver class
 *
 * Abstrac class for all solvers (MCM and MWM solvers, as given in
 * ../mcmsolver and ../mwmsolver).
 *
 */
class Solver
{
public:
    Solver();
    ~Solver();

    virtual void* solve(void* input, Timing* timer, RunParams* rp) = 0;

    virtual int get_input_type() = 0;

    virtual std::string get_solver_name() = 0;

    virtual std::string get_problem_name() = 0;

    virtual void delete_output(void* output) = 0;

    virtual void delete_input(void* input) = 0;

    virtual bool is_result_correct(void* truth, void* computed, RunParams* rp) = 0;

    virtual double get_guarantee(RunParams* rp) = 0;

    virtual double get_min_epsilon();

    virtual double get_min_delta();


    static const int INPUT_TYPE_UNWEIGHTED = 1;
    static const int INPUT_TYPE_WEIGHTED = 2;
    static const int INPUT_TYPE_DYNAMIC_UNWEIGHTED = 3;
    static const int INPUT_TYPE_DYNAMIC_WEIGHTED = 4;

};

#endif
