
#include "bitvector.hpp"


BitVector::BitVector(uint32_t nelems)
{
    this->size = (nelems >> 3) + 1;
    this->array = (uint8_t*) calloc( this->size, sizeof(uint8_t) );
}

BitVector::~BitVector()
{
   free(this->array);
}

uint8_t BitVector::operator[] (const uint32_t& pos) const
{
    return this->get(pos);
}

uint8_t BitVector::get (const uint32_t& pos) const
{
    uint32_t pos_array_index = (pos >> 3);
    uint32_t pos_within = pos % 8;

    uint8_t val  = this->array[pos_array_index];
    val = (val >> pos_within) & 1;
    return val;
}

void BitVector::set (const uint32_t& pos, const bool& bit)
{
    uint32_t pos_array_index = (pos >> 3);
    uint32_t pos_within = pos % 8;

    if(bit)
    {
        uint8_t val = this->array[pos_array_index] | ( 1 << pos_within );
        this->array[pos_array_index] = val;
    }
    else
    {
        uint8_t val = this->array[pos_array_index] & ( ~( 1 << pos_within ));
        this->array[pos_array_index] = val;
    }
}

void BitVector::set_true(const uint32_t&pos)
{
    uint32_t pos_array_index = (pos >> 3);
    uint32_t pos_within = pos % 8;
    uint8_t val_found = this->array[pos_array_index];
    uint8_t val =  val_found | ( (uint8_t)1 << (uint8_t)pos_within );
    this->array[pos_array_index] = val;
}

uint8_t BitVector::get_and_set_true(const uint32_t& pos)
{
    uint32_t pos_array_index = (pos >> 3);
    uint32_t pos_within = pos % 8;

    uint8_t val  = this->array[pos_array_index];
    this->array[pos_array_index] = val | ( 1 << pos_within );

    val = (val >> pos_within) & 1;
    return val;
}
