
#ifndef __GRAPH_H__
#define __GRAPH_H__

#include <cstdint>

#include <iostream>
#include <string>
#include <vector>
#include <unordered_map>
#include <unordered_set>

struct Edge
{
    int32_t u; // first id of the vertex
    int32_t v; // second id of the vertex
};
typedef struct Edge Edge;

struct EdgeList
{
        int32_t n; // number of distinct vertices
        int32_t m; // number of edges (so the lenght of the list)
        std::vector<Edge> * edge; // pointer to the list of edges
};
typedef struct EdgeList EdgeList;


struct WEdge
{
	int32_t u; // first id of the vertex
	int32_t v; // second id of the vertex
	int32_t weight; // edge weight
};
typedef struct WEdge WEdge;

struct WEdgeHash {
   size_t operator() (const WEdge &wedge) const {
    return wedge.u + 2*wedge.v + wedge.weight;
   }
};

inline bool operator == (WEdge const& lhs, WEdge const& rhs)
{
    return (lhs.u == rhs.u) &&
           (lhs.v == rhs.v) &&
            (lhs.weight == rhs.weight);
}

struct EdgeHash {
   size_t operator() (const Edge &wedge) const {
    return wedge.u + 2*wedge.v;
   }
};

inline bool operator == (Edge const& lhs, Edge const& rhs)
{
    return (lhs.u == rhs.u) &&
           (lhs.v == rhs.v);
}

inline bool operator<(const Edge& lhs, const Edge& rhs)
{
  return (lhs.u < rhs.u) || ( lhs.u == rhs.u && lhs.v < rhs.v);
}

struct WEdgeList
{
	int32_t n; // number of distinct vertices
	int32_t m; // number of edges (so the lenght of the list)
        uint32_t max_w; // maximum edge weight
        std::vector<WEdge> * wedge; // pointer to the list of weighted edges
};
typedef struct WEdgeList WEdgeList;

struct RunParams
{
    uint32_t n;
    uint32_t m;
    uint32_t rep;
    uint32_t max_w;
    bool rand_weights;
    double epsilon;
    double delta;
    int nthreads;
    uint32_t num_levels;
    uint32_t k;
};
typedef struct RunParams RunParams;

#endif

