
#ifndef __TIMING_HPP__
#define __TIMING_HPP__

#include <string>
#include <utility>
#include <vector>
#include <sys/time.h>
#include <iostream>
#include <fstream>

#include "../util/util.hpp"
#include "tsc_x86.h"


/**
 * @brief The Timing class
 *
 * The timing class is used to write key-value pairs into a file.
 * That way we can store performance-relevant information for further processing (for example,
 * to store the elapsed time for an algorithm or a partial time in an algorithm)
 */
class Timing
{

public:
    Timing(RunParams* rp, std::string file_path, std::string method_name);
    ~Timing();

    void setInt(std::string key, int32_t value);
    void setUInt(std::string key, uint32_t value);
    void setULong(std::string key, uint64_t value);
    void setULong(std::string key, uint64_t value, bool unique);
    void setDouble(std::string key, double value);
    void setString(std::string key, std::string value);

    void startAllRep();
    void endAllRep();

    void startRep();
    void endRep();

    uint32_t startTiming();
    void endTiming(uint32_t identifier, std::string timing_name, bool unique);
    void endTimingSetup(uint32_t identifier);
    void endTimingTeardown(uint32_t identifier);
    void endTimingProcessingLoop(uint32_t identifier);
    void endTimingMergeLoop(uint32_t identifier);
    void endTimingMainProcessing(uint32_t identifier);

    void writeFile(std::string folder_path);

    static std::string get_output_folder(std::string folder_path, std::string problem_name);

    uint32_t get_repetition();

private:
    double get_time();
    long int get_time_ms();

    std::vector<std::pair<std::string, std::string>> map;
    std::string output_file_name;

    // timers globally
    uint32_t timing_id_rep;
    uint32_t timing_id_global;
    uint32_t num_rep;

    std::vector<std::pair<double, myInt64>> timings;

};

#endif
