/**

  Helper to read tsc (time stamp counter).
  Code taken from How to Write Fast Numerical Code.
  http://www.inf.ethz.ch/personal/markusp/teaching/263-2300-ETH-spring17/homeworks/hw02files/blocker.zip

  Alternatively, use a high precision C++ timer, such as
  http://www.cplusplus.com/reference/chrono/high_resolution_clock/now/

*/


#ifndef __TSC_X86__
#define __TSC_X86__

/* ==================== GNU C and possibly other UNIX compilers ===================== */
	#ifndef _WIN64

	#if defined(__GNUC__) || defined(__linux__)
		#define VOLATILE __volatile__
		#define ASM __asm__
	#else
		/* if we're neither compiling with gcc or under linux, we can hope
		 * the following lines work, they probably won't */
		#define ASM asm
		#define VOLATILE 
	#endif

	#define myInt64 unsigned long long
	#define INT32 unsigned int

/* ======================== WIN32 ======================= */
#else

	#define myInt64 signed __int64
	#define INT32 unsigned __int32

#endif

/* This is the RDTSC timer.
 * RDTSC is an instruction on several Intel and compatible CPUs that Reads the
 * Time Stamp Counter. The Intel manuals contain more information.
 */


#define COUNTER_LO(a) ((a).int32.lo)
#define COUNTER_HI(a) ((a).int32.hi)
#define COUNTER_VAL(a) ((a).int64)

#define COUNTER(a) \
	((unsigned long long)COUNTER_VAL(a))

#define COUNTER_DIFF(a,b) \
	(COUNTER(a)-COUNTER(b))

/* ==================== GNU C and possibly other UNIX compilers ===================== */
#ifndef _WIN64

	typedef union
	{       myInt64 int64;
		    struct {INT32 lo, hi;} int32;
	} tsc_counter;

  #define RDTSC_UTIL(cpu_c) \
	  ASM VOLATILE ("rdtsc" : "=a" ((cpu_c).int32.lo), "=d"((cpu_c).int32.hi))
    #define CPUID_UTIL() \
		ASM VOLATILE ("cpuid" : : "a" (0) : "bx", "cx", "dx" )

/* ======================== WIN32 ======================= */
#else

	typedef union
	{       myInt64 int64;
			struct {INT32 lo, hi;} int32;
	} tsc_counter;

    #define RDTSC_UTIL(cpu_c)   \
	{       __asm rdtsc    \
			__asm mov (cpu_c).int32.lo,eax  \
			__asm mov (cpu_c).int32.hi,edx  \
	}

    #define CPUID_UTIL() \
	{ \
		__asm mov eax, 0 \
		__asm cpuid \
	}

#endif


void init_tsc();

myInt64 start_tsc(void);

myInt64 stop_tsc(myInt64 start);

#endif


