
#include "util.hpp"

void Util::split(const std::string &s, std::string delim, std::vector<std::string> *result) {

    if(result == nullptr)
    {
        return;
    }

    std::size_t prev = 0, pos;
    while ((pos = s.find_first_of(delim, prev)) != std::string::npos)
    {
        if (pos > prev)
        {
            result->push_back(s.substr(prev, pos-prev));
        }
        prev = pos+1;
    }

    if (prev < s.length())
    {
        result->push_back(s.substr(prev, std::string::npos));
    }
}

std::vector<uint32_t> Util::str_to_uint(std::vector<std::string> vec)
{
    std::vector<uint32_t> result;
    for(auto it = vec.begin(); it != vec.end(); ++it)
    {
        uint32_t value = std::strtoul( (*it).c_str(), NULL, 0 );
        result.push_back(value);
    }
    return result;
}

std::vector<uint32_t> Util::split_whitespace_to_uint(const std::string &s)
{
    std::vector<std::string> elems;
    Util::split(s, " \t", &elems);

    std::vector<uint32_t> result = Util::str_to_uint(elems);
    return result;
}


bool Util::has_suffix(const std::string &str, const std::string &suffix)
{
    return str.size() >= suffix.size() && str.compare(str.size() - suffix.size(), suffix.size(), suffix) == 0;
}


void Util::print_edge_list(WEdgeList *edgeList)
{
    Util::print_edge_list(edgeList, 0, 1);
}

void Util::print_edge_list(WEdgeList *edgeList, uint32_t max_i, uint32_t k)
{
    if(k == 0)
    {
        k = 1;
    }

    puts("Write to cpu.input.txt");
    std::ofstream newFile1("cpu.input.txt");

    std::cout << "WEdgeList: n="<< edgeList->n << ", m="<< edgeList->m << std::endl;
    for(uint32_t i = 0; i < edgeList->m && ( max_i == 0 || i < max_i); i++)
    {
        newFile1 << "Edge[" << i << "]: epoch="<<  ( ( (*edgeList->wedge)[i].u ) -1 )/ k << ", u=" << (*edgeList->wedge)[i].u << ", v=" << (*edgeList->wedge)[i].v << ", w="<< (*edgeList->wedge)[i].weight << std::endl;
    }
    newFile1.close();

    if(max_i != 0)
    {
        for(uint32_t i = std::max( edgeList->m - max_i, (uint32_t) 0); i < edgeList->m; i++ )
        {
            std::cout << "Edge[" << i << "]: u=" << (*edgeList->wedge)[i].u << ", v="<< (*edgeList->wedge)[i].v << ", w=" << (*edgeList->wedge)[i].weight << std::endl;
        }
    }

}

void Util::print_edge_list(lemon::ListGraph* g)
{
    Util::print_edge_list(g, nullptr, 0);
}

void Util::print_edge_list(lemon::ListGraph *g, lemon::ListGraph::EdgeMap<int>* cost, uint32_t max_i)
{
    puts("WEdgeList"); //  n=%d, m=%d

    typedef lemon::ListGraph::EdgeIt EdgeIt;
    using lemon::INVALID;

    uint32_t counter = 0;
    for (EdgeIt i(*g); i!=INVALID && (max_i == 0 || counter < max_i ); ++i)
    {
        std::string weight = "unkown";
        if(cost != nullptr)
        {
            weight = std::to_string( (*cost)[i]);
        }

        std::cout << "WEdge: u=" << g->id(g->u(i)) << ", v=" << g->id(g->v(i)) <<", w="<< weight << std::endl;
        counter++;
    }
}

void Util::print_edge_list(EdgeList* edgeList, uint32_t max_i)
{
    std::cout << "EdgeList: n=" << edgeList->n << ", m=" << edgeList->m << std::endl;
    for(uint32_t i = 0; i < edgeList->m && ( max_i == 0 || i < max_i); i++)
    {
        std::cout << "Edge["<< i <<"]: u="<< (*edgeList->edge)[i].u << ", v=" << (*edgeList->edge)[i].v << std::endl;
    }

    if(max_i != 0)
    {
        for(uint32_t i = std::max( edgeList->m - max_i,(uint32_t)  0); i < edgeList->m; i++ )
        {
            std::cout << "Edge["<< i <<"]: u="<< (*edgeList->edge)[i].u << ", v=" << (*edgeList->edge)[i].v << std::endl;
        }
    }
}



// from https://stackoverflow.com/questions/997946/how-to-get-current-time-and-date-in-c
std::string Util::currentDateTime() {
    time_t     now = time(0);
    struct tm  tstruct;
    char       buf[80];
    tstruct = *localtime(&now);

    // http://en.cppreference.com/w/cpp/chrono/c/strftime
    // for more information about date/time format
    strftime(buf, sizeof(buf), "%Y_%m_%d_%H_%M_%S", &tstruct);

    return buf;
}


// from https://stackoverflow.com/questions/8520560/get-a-file-name-from-a-path
std::string Util::getFileName(std::string file_path)
{
    std::string base_filename = file_path.substr(file_path.find_last_of("/\\") + 1);
    return base_filename;
}


bool Util::createFolder(std::string path)
{
    char tmp[PATH_MAX];
    char *p = nullptr;
    size_t len;

    snprintf(tmp, sizeof(tmp),"%s",path.c_str());
    len = strlen(tmp);
    if(tmp[len - 1] == '/')
    {
        tmp[len - 1] = 0;
    }
    for(p = tmp + 1; *p; p++)
    {
        if(*p == '/')
        {
            *p = 0;
            if( mkdir(tmp, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH) == -1)
            {
                if( !( errno == EEXIST) )
                {
                    return false;
                }
            }

            *p = '/';
        }
    }
    if( mkdir(tmp, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH) == -1)
    {
        if( !( errno == EEXIST) )
        {
            return false;
        }
    }

    return true;
}

bool Util::cleanFolder(std::string path)
{
    DIR *theFolder = opendir(path.c_str());
    struct dirent *next_file;
    char filepath[PATH_MAX];

    while ( (next_file = readdir(theFolder)) != NULL )
    {
        if (0 == strcmp(next_file->d_name, ".") || 0 == strcmp(next_file->d_name, ".."))
        {
            continue;
        }

        // build the path for each file in the folder
        sprintf(filepath, "%s/%s", path.c_str(), next_file->d_name);
        std::cout << "Remove: "<< filepath<< std::endl;
        remove(filepath);
    }
    closedir(theFolder);

    return true;
}



// from http://forum.codecall.net/topic/68935-how-to-test-if-file-or-directory/
bool Util::is_file(const char* path) {
    struct stat buf;
    stat(path, &buf);
    return S_ISREG(buf.st_mode);
}

// from http://forum.codecall.net/topic/68935-how-to-test-if-file-or-directory/
bool Util::is_dir(const char* path) {
    struct stat buf;
    stat(path, &buf);
    return S_ISDIR(buf.st_mode);
}

void Util::edge_list_to_lemon_graph(EdgeList* input, lemon::ListGraph *g)
{
    std::vector<lemon::ListGraph::Node> node_list;
    std::vector<lemon::ListGraph::Edge> edge_list;

    Util::edge_list_to_lemon_graph(input, g, &node_list, &edge_list);
}

void Util::edge_list_to_lemon_graph(EdgeList* input, lemon::ListGraph *g, std::vector<lemon::ListGraph::Node> * node_list, std::vector<lemon::ListGraph::Edge>* edge_list)
{
    for(uint32_t i = 0; i < input->n; i++)
    {
        lemon::ListGraph::Node n = g->addNode();
        node_list->push_back(n);
    }

    for(uint32_t i = 0; i < input->m; i++)
    {
        Edge e = input->edge->at(i);
        lemon::ListGraph::Node a = node_list->at(e.u - 1); // due to 1-based indexing
        lemon::ListGraph::Node b = node_list->at(e.v-1); // due to 1-based indexing

        lemon::ListGraph::Edge ee = g->addEdge(a, b);
        edge_list->push_back( ee );
    }
}


void Util::wedge_list_to_lemon_graph(WEdgeList * input, lemon::ListGraph *g, lemon::ListGraph::EdgeMap<int> *cost)
{

    std::vector<lemon::ListGraph::Node> node_list;
    std::vector<lemon::ListGraph::Edge> edge_list;

    for(uint32_t i = 0; i < input->n; i++)
    {
        lemon::ListGraph::Node n = g->addNode();
        node_list.push_back(n);
    }

    for(uint32_t i = 0; i < input->m; i++)
    {
        WEdge e = input->wedge->at(i);
        lemon::ListGraph::Node a = node_list[e.u - 1]; // due to 1-based indexing
        lemon::ListGraph::Node b = node_list[e.v-1]; // due to 1-based indexing

        lemon::ListGraph::Edge ee = g->addEdge(a, b);
        edge_list.push_back( ee );
    }


    for(uint32_t i = 0; i < input->m; i++)
    {
        lemon::ListGraph::Edge key = edge_list[i] ;
        int value =(int)input->wedge->at(i).weight;
        (*cost)[key] = value; //.set( key, value);
    }
}

// from https://stackoverflow.com/questions/3418231/replace-part-of-a-string-with-another-string
bool Util::replace(std::string& str, const std::string& from, const std::string& to)
{
    size_t start_pos = str.find(from);
    if(start_pos == std::string::npos)
    {
        return false;
    }

    str.replace(start_pos, from.length(), to);
    return true;
}


uint32_t* Util::convert_to_cache_line_format(WEdgeList* wedge, uint64_t* num_cache_lines)
{
    *num_cache_lines = (uint64_t) ceil( (double)wedge->m / (double)5.0f);

    uint32_t *result = (uint32_t*)calloc(512/8, (*num_cache_lines) );

    uint64_t k_4byte_counter = 0;
    for(uint64_t i = 0; i < wedge->m; i++)
    {
        if(i % 5*3 == 0 && i != 0) // every 15 edges, put an empty part
        {
            // put empty element - do nothing
            k_4byte_counter++;
        }

        result[k_4byte_counter] = wedge->wedge->at(i).u;
        result[k_4byte_counter +1] = wedge->wedge->at(i).v;
        result[k_4byte_counter +2] = wedge->wedge->at(i).weight;

        k_4byte_counter += 3;
    }
    return result;
}

WEdgeList* Util::convert_cache_line_format_to_wedgelist(uint32_t* cache_line, uint64_t num_cache_lines )
{
    WEdgeList* result = (WEdgeList*) malloc(sizeof(WEdgeList));
    result->wedge = new std::vector<WEdge>();

    for(uint64_t i = 0; i < num_cache_lines; i++)
    {
        uint32_t* cl_start = cache_line+i*512/32;
        for(int k = 0;  k < 5; k++)
        {
            if( *(cl_start+k*3) != *(cl_start+k*3+1 ) )
            {
                WEdge w;
                w.u = *(cl_start+k*3);
                w.v = *(cl_start+k*3+1 );
                w.weight = *(cl_start+k*3+2);
                result->wedge->push_back(w);
            }
        }
    }
    result->m = result->wedge->size();
    return result;
}

bool compareWEdge(const WEdge &a, const WEdge &b)
{
    return ( a.u < b.u || ( a.u == b.u && a.v < b.v) );
}

int Util::convert_to_csr_fpga_format(WEdgeList* p_wedge, uint32_t** p_ptrs, uint64_t* p_num_cl_ptr, uint32_t** p_edges, uint64_t* p_num_cl_edges)
{
    if( p_wedge->m == 0 || p_wedge->n == 0)
    {
        *p_num_cl_edges= 1;
        *p_num_cl_ptr = 1;
        *p_edges = (uint32_t*) calloc(512/8, *p_num_cl_edges);
        *p_ptrs = (uint32_t*) calloc(512/8, *p_num_cl_ptr);
        return 0;
    }

    std::sort(p_wedge->wedge->begin(), p_wedge->wedge->end(), compareWEdge);

    std::vector<uint32_t> ptrs;

    *p_num_cl_edges = (uint64_t)  p_wedge->wedge->size()/ (uint64_t) 8 + 1;
    uint32_t* edges = (uint32_t*) calloc(512/8, *p_num_cl_edges);

    std::cout << "allocates cache-lines for edges="<< *p_num_cl_edges << std::endl;

    uint64_t u = 1;
    uint64_t count = 0;
    uint64_t total_count = 0;

    ptrs.push_back(0); // the start infos for the very first entry
    ptrs.push_back(0);
    for(int i = 0; i < p_wedge->wedge->size(); i++)
    {
        edges[2*i] = p_wedge->wedge->at(i).v;
        edges[2*i+1] = p_wedge->wedge->at(i).weight;

        if(u != p_wedge->wedge->at(i).u)
        {
            do
            {
                ptrs.push_back(count);

                {
                    if( count == 0)
                    {
                        // std::cout << "count = 0 at i = "<< i<<std::endl;
                    }
                    ptrs.push_back( i /8);
                    ptrs.push_back( i - ( ( i/8 ) * 8)  );
                }

                total_count= total_count  + count;
                count = 0;
                u = u + 1 ;
            } while( p_wedge->wedge->at(i).u != u);
        }
        count++;
    }
    ptrs.push_back(count); // the count for the last entry
    total_count= total_count + count;
    std::cout << "total_count (num_edges)="<< total_count<<""<< std::endl;

    // check if w need to push more (empty) pointers, if there are some left after the last "n"
    do
    {
        ptrs.push_back(0);
        ptrs.push_back(0);
        ptrs.push_back(0);
        u = u + 1;
    }while(u != p_wedge->n + 1);


    *p_edges = edges;

    *p_num_cl_ptr =  ptrs.size() / (uint64_t)(3.0*5.0) +1;
    *p_ptrs = (uint32_t*)calloc((*p_num_cl_ptr) , 512/8);

    std::cout << "allocated cache-lines for ptrs: "<< *p_num_cl_ptr << std::endl;

    uint64_t k_4byte_counter = 0;
    for(uint64_t i = 0; i < ptrs.size()/3; i++)
    {
        if(i % 5*3 == 0 && i != 0) // every 15 edges, put an empty part
        {
            // put empty element - do nothing
            k_4byte_counter++;
        }

        (*p_ptrs)[k_4byte_counter] = ptrs.at(i*3);
        (*p_ptrs)[k_4byte_counter +1] = ptrs.at(i*3+1);
        (*p_ptrs)[k_4byte_counter +2] = ptrs.at(i*3+2);

        k_4byte_counter += 3;
    }

    return 0;
}


WEdgeList* Util::convert_wedgelist_to_k_merged_wedgelist(WEdgeList* list, uint64_t k)
{
    WEdgeList* result = (WEdgeList*)malloc(sizeof(WEdgeList));
    result->m = list->m;
    result->n = list->n;
    result->wedge = new std::vector<WEdge>();
    result->max_w = list->max_w;
    std::cout << "start sort"<< std::endl;
    std::sort(list->wedge->begin(), list->wedge->end(), compareWEdge);
    std::cout << "end sort" <<std::endl;

    std::cout << "generate the list of pointers. Given list: list->m="<< list->m << std::endl;
    std::vector<uint32_t> pointers;

    uint64_t current_u = 1;
    uint64_t current_start = 0;
    for(uint64_t i = 0; i < list->m; i++)
    {
        // if there is a transition, or it is the last edges, then we must add a pointer
        if( current_u != list->wedge->at(i).u  )
        {
            do {
                // generate the pointer for this row ( and empty pointers too, if we need to)
                pointers.push_back( current_start);
                pointers.push_back(i);

                current_u = current_u + 1;
                current_start = i;
            } while(list->wedge->at(i).u != current_u );
        }
    }


    //add the last pointer and more pointers, if some are missing at the end
    pointers.push_back(current_start);
    pointers.push_back(list->m);

    std::cout << "generate ordering with pointers count: " << pointers.size() << std::endl;
    uint64_t num_epochs = (uint64_t)  list->n /(uint64_t)k + 1;
    for(uint64_t i = 0; i < num_epochs; i++)
    {
        while(true)
        {
            // determine the smallest "v" value
            WEdge we;
            we.v = std::numeric_limits<int32_t>::max();
            uint64_t selected_pointer = 0;
            for(uint64_t j = 0; j < k && 2*( i*k+j) + 1  < pointers.size(); j++)
            {
                // test if this pointer is valid
                if( pointers[ 2*( i*k+j ) ] < pointers[2*( i*k+j) + 1 ] )
                {
                    if( we.v >  list->wedge->at( pointers[2* ( i*k+j ) ]).v )
                    {
                        we = list->wedge->at(  pointers[2* ( i*k+j )] );
                        selected_pointer = 2* ( i*k+j);
                    }
                }
            }
            // go to next epoch, if no v found
            if( we.v == std::numeric_limits<int32_t>::max())
            {
                break;
            }
            pointers[selected_pointer ] =  pointers[ selected_pointer] +  1;
            result->wedge-> push_back(we   );
        }
    }

    return result;
}


void Util::sanitize_wedge_list(WEdgeList* list)
{
    bool has_null_vertex = false;
    for(uint32_t i = 0; i < list->wedge->size(); i++)
    {
        WEdge e = list->wedge->at(i);
        if(e.u == 0 || e.v == 0)
        {
            has_null_vertex = true;
            std::cout << "detected a vertex with index 0: i="<< i << ", e.u="<< e.u<<", e.v="<< e.v << std::endl;
            break;
        }
    }

    if( has_null_vertex )
    {
        for(uint32_t i = 0; i < list->wedge->size(); i++)
        {
            list->wedge->at(i).u = list->wedge->at(i).u +1;
            list->wedge->at(i).v = list->wedge->at(i).v +1;
        }
    }
    list->n = list->n + 1;

    uint32_t count = 0;
    for(uint32_t i = 0; i < list->wedge->size(); i++)
    {
        if( list->wedge->at(i).u > list->wedge->at(i).v)
        {
            count ++;
            uint32_t temp = list->wedge->at(i).u;
            list->wedge->at(i).u = list->wedge->at(i).v;
            list->wedge->at(i).v = temp;
        }
    }
    std::cout << "detected "<< count<<" many edges which must be switched"<< std::endl;


}



