
#ifndef __UTIL_HPP__
#define __UTIL_HPP__

#include <cstdint>
#include <string>
#include <vector>
#include <sstream>
#include <cstdlib>
#include <algorithm>

#include <string>
#include <vector>
#include <fstream>
#include <iostream>
#include <stdexcept>

#include <unistd.h>
#include <limits>

#include <stdio.h>
#include <dirent.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <limits.h>

#include <stdio.h>
#include <string.h>


#include <lemon/list_graph.h>
#include <lemon/concepts/graph.h>
#include <lemon/core.h>
#include <lemon/matching.h>
#include <lemon/list_graph.h>


#include "graph.hpp"

/**
 * @brief The Util class
 *
 * Contains general methods, which can be used by any other class and method.
 *
 * Not all methods are self-implemented. See this header and the according .cpp file for credit
 *
 */
class Util
{
public:
    /**
     * @brief split Splits values in s by delim and stores it in the provided vector result
     * @param s
     * @param delim
     * @param result
     */
    static void split(const std::string &s, std::string delim, std::vector<std::string>* result);

    /**
     * @brief str_to_int Converts an array of strings to an array of integers
     * @param vec
     * @return
     */
    static std::vector<uint32_t> str_to_uint(std::vector<std::string> vec);

    /**
     * @brief split_whitespace_to_int Splits the values in s by whitespace, and returns the values as integers
     * @param s
     * @return
     */
    static std::vector<uint32_t> split_whitespace_to_uint(const std::string &s);

    /**
     * @brief has_suffix Tests if a string ends with the given suffix
     * Credits to: Dietrich Epp at https://stackoverflow.com/questions/20446201/how-to-check-if-string-ends-with-txt
     *
     * @param str
     * @param suffix
     * @return
     */
    static bool has_suffix(const std::string &str, const std::string &suffix);


    static void print_edge_list(WEdgeList* edgeList);

    static void print_edge_list(WEdgeList* edgeList, uint32_t max_i, uint32_t k);

    static void print_edge_list(EdgeList* edgeList, uint32_t max_i);

    static void print_edge_list(lemon::ListGraph* g);

    static void print_edge_list(lemon::ListGraph* g, lemon::ListGraph::EdgeMap<int>* cost, uint32_t);

    static std::string currentDateTime();

    static std::string getFileName(std::string file_path);

    static bool createFolder(std::string path);

    static bool cleanFolder(std::string path);

    static bool is_file(const char* path);

    static bool is_dir(const char* path);

    static void edge_list_to_lemon_graph(EdgeList* input, lemon::ListGraph *g);

    static void edge_list_to_lemon_graph(EdgeList* input, lemon::ListGraph *g, std::vector<lemon::ListGraph::Node> * node_list, std::vector<lemon::ListGraph::Edge>* edge_list);

    static void wedge_list_to_lemon_graph(WEdgeList * input, lemon::ListGraph *g, lemon::ListGraph::EdgeMap<int> *cost);

    static bool replace(std::string& str, const std::string& from, const std::string& to);

    static uint32_t* convert_to_cache_line_format(WEdgeList* wedge, uint64_t* num_cache_lines);

    static WEdgeList* convert_cache_line_format_to_wedgelist(uint32_t*cache_line, uint64_t num_cache_lines );

    static int convert_to_csr_fpga_format(WEdgeList* p_wedge, uint32_t** p_ptrs, uint64_t* p_num_cl_ptr, uint32_t** p_edges, uint64_t* p_num_cl_edges);

    static WEdgeList* convert_wedgelist_to_k_merged_wedgelist(WEdgeList* list, uint64_t k);

    static void sanitize_wedge_list(WEdgeList* list);

};


#endif
